# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2026 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import os
import pathlib
import subprocess
import sys
import winreg
import xml.etree.ElementTree as ET
import testset as testset
import testhelper as testhelper

class IntegrationTests:
    def __init__(self, installation_type="normal", test_type="install"):
        self.testhelper = testhelper.TestHelper()
        self.installation_type = installation_type
        self.test_type = test_type
        self.result = True
        self.result &= self.test_version()
        self.result &= self.test_files_exist()
        self.result &= self.test_registry_records_exist()
        self.result &= self.test_login_token_file()
        if self.installation_type == "service":
            self.result &= self.test_service_exists()
            self.result &= self.test_service_users_exist()
            self.result &= self.test_service_groups_and_memberships()
        if self.installation_type == "test_acct_mgr_login":
            self.result &= self.test_acct_mgr_login()
        if self.installation_type == "test_client_auth_file":
            self.result &= self.test_client_auth_file()
            if self.test_type == "install":
                self.result &= self.test_boinc_master_user_only_exists()
        if self.installation_type == "test_proj_init_file":
            self.result &= self.test_proj_init_file()

    def _get_test_executable_file_path(self, filename):
        return pathlib.Path("C:\\Program Files\\BOINC\\") / filename

    def _get_test_data_file_path(self, filename):
        return pathlib.Path("C:\\ProgramData\\BOINC\\") / filename

    def _get_file_version(self, filename):
        return self.testhelper.get_version_from_string(os.popen(("\"{app}\" --version").format(app=self._get_test_executable_file_path(filename))).read().strip())

    def _get_value_from_registry(self, key, value):
        try:
            hive, subkey = key.split("\\", 1)
            hives = {
                "HKEY_CLASSES_ROOT": winreg.HKEY_CLASSES_ROOT,
                "HKEY_CURRENT_USER": winreg.HKEY_CURRENT_USER,
                "HKEY_LOCAL_MACHINE": winreg.HKEY_LOCAL_MACHINE,
                "HKEY_USERS": winreg.HKEY_USERS,
                "HKEY_CURRENT_CONFIG": winreg.HKEY_CURRENT_CONFIG
            }
            with winreg.OpenKey(hives[hive], subkey) as key:
                result, _ = winreg.QueryValueEx(key, value)
                return result
        except Exception as e:
            print(f"Error accessing registry: {e}")
            return None

    def _check_service_exists(self, service_name):
        try:
            result = subprocess.run(
                ["sc", "query", service_name],
                capture_output=True,
                text=True,
                check=False
            )
            return result.returncode == 0
        except Exception as e:
            print(f"Error checking service: {e}")
            return False

    def _check_user_exists(self, username):
        try:
            result = subprocess.run(
                ["net", "user", username],
                capture_output=True,
                text=True,
                check=False
            )
            return result.returncode == 0
        except Exception as e:
            print(f"Error checking user: {e}")
            return False

    def _check_group_exists(self, groupname, username=None):
        try:
            result = subprocess.run(
                ["net", "localgroup", groupname],
                capture_output=True,
                text=True,
                check=False
            )
            if username is not None:
                if result.returncode != 0:
                    return False
                return username in result.stdout
            # Just check if the group exists
            return result.returncode == 0
        except Exception as e:
            print(f"Error checking group: {e}")
            return False

    def test_version(self):
        ts = testset.TestSet("Test version is correct")
        current_version = self.testhelper.get_current_version_number()
        ts.expect_not_equal("", current_version, "Test current version could be read from the 'version.h' file")
        ts.expect_equal(current_version, self._get_file_version("boinc.exe"), "Test 'boinc.exe' version is correctly set")
        ts.expect_equal(current_version, self._get_file_version("boinccmd.exe"), "Test 'boinccmd.exe' version is correctly set")
        ts.expect_equal(current_version, self._get_value_from_registry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Space Sciences Laboratory, U.C. Berkeley\\BOINC Setup\\", "UpgradingTo"), "Test 'UpgradingTo' registry record exists and is set correctly")
        return ts.result()

    def test_files_exist(self):
        ts = testset.TestSet("Test files exist")
        ts.expect_true(os.path.exists(self._get_test_executable_file_path("boinc.exe")), "Test 'boinc.exe' file exists in 'C:\\Program Files\\BOINC\\'")
        ts.expect_true(os.path.exists(self._get_test_executable_file_path("boinccmd.exe")), "Test 'boinccmd.exe' file exists in 'C:\\Program Files\\BOINC\\'")
        ts.expect_true(os.path.exists(self._get_test_executable_file_path("boincmgr.exe")), "Test 'boincmgr.exe' file exists in 'C:\\Program Files\\BOINC\\'")
        ts.expect_true(os.path.exists(self._get_test_executable_file_path("boincscr.exe")), "Test 'boincscr.exe' file exists in 'C:\\Program Files\\BOINC\\'")
        ts.expect_true(os.path.exists(self._get_test_executable_file_path("boinctray.exe")), "Test 'boinctray.exe' file exists in 'C:\\Program Files\\BOINC\\'")
        ts.expect_true(os.path.exists(self._get_test_data_file_path("all_projects_list.xml")), "Test 'all_projects_list.xml' file exists in 'C:\\ProgramData\\BOINC\\'")
        return ts.result()

    def test_registry_records_exist(self):
        ts = testset.TestSet("Test registry records exist")
        ts.expect_equal("C:\\ProgramData\\BOINC\\", self._get_value_from_registry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Space Sciences Laboratory, U.C. Berkeley\\BOINC Setup\\", "DATADIR"), "Test 'DATADIR' registry record exists and is set correctly")
        ts.expect_equal("C:\\Program Files\\BOINC\\", self._get_value_from_registry("HKEY_LOCAL_MACHINE\\SOFTWARE\\Space Sciences Laboratory, U.C. Berkeley\\BOINC Setup\\", "INSTALLDIR"), "Test 'INSTALLDIR' registry record exists and is set correctly")
        return ts.result()

    def test_service_exists(self):
        ts = testset.TestSet("Test BOINC service exists")
        ts.expect_true(self._check_service_exists("boinc"), "Test 'boinc' service exists")
        return ts.result()

    def test_service_users_exist(self):
        ts = testset.TestSet("Test BOINC service users exist")
        ts.expect_true(self._check_user_exists("boinc_master"), "Test 'boinc_master' user exists")
        ts.expect_true(self._check_user_exists("boinc_project"), "Test 'boinc_project' user exists")
        return ts.result()

    def test_boinc_master_user_only_exists(self):
        ts = testset.TestSet("Test 'boinc_master' user only exists")
        ts.expect_true(self._check_user_exists("boinc_master"), "Test 'boinc_master' user exists")
        ts.expect_false(self._check_user_exists("boinc_project"), "Test 'boinc_project' user does not exist")
        return ts.result()

    def test_service_groups_and_memberships(self):
        ts = testset.TestSet("Test BOINC service groups and memberships")
        ts.expect_true(self._check_group_exists("boinc_admins"), "Test 'boinc_admins' group exists")
        ts.expect_true(self._check_group_exists("boinc_projects"), "Test 'boinc_projects' group exists")
        ts.expect_true(self._check_group_exists("boinc_users"), "Test 'boinc_users' group exists")
        ts.expect_true(self._check_group_exists("boinc_admins", "boinc_master"), "Test 'boinc_master' user is a member of 'boinc_admins' group")
        ts.expect_true(self._check_group_exists("boinc_projects", "boinc_project"), "Test 'boinc_project' user is a member of 'boinc_projects' group")
        return ts.result()

    def test_acct_mgr_login(self):
        ts = testset.TestSet("Test account manager login file")
        acct_mgr_file = self._get_test_data_file_path("acct_mgr_login.xml")
        ts.expect_true(os.path.exists(acct_mgr_file), "Test 'acct_mgr_login.xml' file exists in 'C:\\ProgramData\\BOINC\\'")

        if os.path.exists(acct_mgr_file):
            try:
                tree = ET.parse(acct_mgr_file)
                root = tree.getroot()

                ts.expect_equal("acct_mgr_login", root.tag, "Test root element is 'acct_mgr_login'")

                login_element = root.find("login")
                password_hash_element = root.find("password_hash")

                ts.expect_true(login_element is not None, "Test 'login' element exists")
                ts.expect_true(password_hash_element is not None, "Test 'password_hash' element exists")

                if login_element is not None:
                    ts.expect_equal("test", login_element.text, "Test 'login' element contains 'test'")

                if password_hash_element is not None:
                    ts.expect_equal("0123456789abcdef", password_hash_element.text, "Test 'password_hash' element contains '0123456789abcdef'")

            except ET.ParseError as e:
                ts.expect_true(False, f"Test XML file is well-formed (Parse error: {e})")
            except Exception as e:
                ts.expect_true(False, f"Test XML file can be processed (Error: {e})")

        return ts.result()

    def test_client_auth_file(self):
        ts = testset.TestSet("Test client auth file")
        client_auth_file = self._get_test_data_file_path("client_auth.xml")
        ts.expect_true(os.path.exists(client_auth_file), "Test 'client_auth.xml' file exists in 'C:\\ProgramData\\BOINC\\'")

        if os.path.exists(client_auth_file):
            try:
                tree = ET.parse(client_auth_file)
                root = tree.getroot()

                ts.expect_equal("client_authorization", root.tag, "Test root element is 'client_authorization'")

                boinc_project_element = root.find("boinc_project")
                ts.expect_true(boinc_project_element is not None, "Test 'boinc_project' element exists")

                if boinc_project_element is not None:
                    username_element = boinc_project_element.find("username")
                    password_element = boinc_project_element.find("password")

                    ts.expect_true(username_element is not None, "Test 'username' element exists")
                    ts.expect_true(password_element is not None, "Test 'password' element exists")

                    if username_element is not None:
                        ts.expect_equal("test_user", username_element.text, "Test 'username' element contains 'test_user'")

                    if password_element is not None:
                        ts.expect_equal("cXdlcnR5MTIzNDU2IUAjJCVe", password_element.text, "Test 'password' element contains 'cXdlcnR5MTIzNDU2IUAjJCVe'")

            except ET.ParseError as e:
                ts.expect_true(False, f"Test XML file is well-formed (Parse error: {e})")
            except Exception as e:
                ts.expect_true(False, f"Test XML file can be processed (Error: {e})")

        return ts.result()

    def test_login_token_file(self):
        ts = testset.TestSet("Test login token file")
        login_token_file = self._get_test_data_file_path("login_token.txt")
        ts.expect_true(os.path.exists(login_token_file), "Test 'login_token.txt' file exists in 'C:\\ProgramData\\BOINC\\'")

        if os.path.exists(login_token_file):
            try:
                with open(login_token_file, 'r') as f:
                    content = f.read()
                    ts.expect_equal("installer_setup.exe", content.strip(), "Test 'login_token.txt' contains 'installer_setup.exe' line")
            except Exception as e:
                ts.expect_true(False, f"Test 'login_token.txt' can be read (Error: {e})")

        return ts.result()

    def test_proj_init_file(self):
        ts = testset.TestSet("Test project init file")
        project_init_file = self._get_test_data_file_path("project_init.xml")
        ts.expect_true(os.path.exists(project_init_file), "Test 'project_init.xml' file exists in 'C:\\ProgramData\\BOINC\\'")

        if os.path.exists(project_init_file):
            try:
                tree = ET.parse(project_init_file)
                root = tree.getroot()

                ts.expect_equal("project_init", root.tag, "Test root element is 'project_init'")

                url_element = root.find("url")
                name_element = root.find("name")
                account_key_element = root.find("account_key")
                embedded_element = root.find("embedded")

                ts.expect_true(url_element is not None, "Test 'url' element exists")
                ts.expect_true(name_element is not None, "Test 'name' element exists")
                ts.expect_true(account_key_element is not None, "Test 'account_key' element exists")
                ts.expect_true(embedded_element is not None, "Test 'embedded' element exists")

                if url_element is not None:
                    ts.expect_equal("https://test.com", url_element.text, "Test 'url' element contains 'https://test.com'")

                if name_element is not None:
                    ts.expect_equal("test", name_element.text, "Test 'name' element contains 'test'")

                if account_key_element is not None:
                    ts.expect_equal("abcdef1234567890", account_key_element.text, "Test 'account_key' element contains 'abcdef1234567890'")

                if embedded_element is not None:
                    ts.expect_equal("0", embedded_element.text, "Test 'embedded' element contains '0'")

            except ET.ParseError as e:
                ts.expect_true(False, f"Test XML file is well-formed (Parse error: {e})")
            except Exception as e:
                ts.expect_true(False, f"Test XML file can be processed (Error: {e})")

        return ts.result()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="BOINC Windows Installer Integration Tests")
    parser.add_argument(
        "--installation-type",
        type=str,
        default="normal",
        help="Installation type: 'normal' (default), 'service', 'test_acct_mgr_login', 'test_client_auth_file', or 'test_proj_init_file'"
    )
    parser.add_argument(
        "--type",
        type=str,
        default="install",
        help="Test type: 'install' (default) or 'upgrade_from_alpha' or 'upgrade_from_stable'"
    )
    args = parser.parse_args()

    if not IntegrationTests(installation_type=args.installation_type, test_type=args.type).result:
        sys.exit(1)
    sys.exit(0)



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2026 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys

linux_client_list = [
    './client/boinc',
    './client/boinccmd',
    './client/scripts/boinc-client.service',
    './client/scripts/boinc-client',
    './client/scripts/boinc.bash',
    './client/scripts/boinc-client.conf',
    './packages/deb/*',
    './packages/generic/36x11-common_xhost-boinc',
    'locale/*/*.mo',
    './win_build/installerv2/redist/all_projects_list.xml'
]

linux_apps_list = [
    './samples/condor/boinc_gahp',
    './samples/example_app/uc2',
    './samples/example_app/ucn',
    './samples/example_app/uc2_graphics',
    './samples/example_app/slide_show',
    './samples/multi_thread/multi_thread*pc-linux-gnu',
    './samples/sleeper/sleeper',
    './samples/vboxmonitor/vboxmonitor',
    './samples/vboxwrapper/vboxwrapper*pc-linux-gnu',
    './samples/worker/worker*pc-linux-gnu',
    './samples/wrapper/wrapper*pc-linux-gnu',
    './samples/openclapp/openclapp',
    './samples/wrappture/wrappture_example',
    './samples/wrappture/fermi',
    './samples/sporadic/sporadic',
    './samples/docker_wrapper/docker_wrapper*pc-linux-gnu',
]

linux_manager_list = [
    './clientgui/boincmgr',
    './clientgui/skins',
    './clientgui/res/boinc.desktop',
    './clientgui/res/boinc.png',
    './clientgui/res/boinc.svg',
    'locale/*/*.mo',
]

mingw_apps_list = [
    './lib/wrapper.exe'
]

mingw_apps_vcpkg_list = [
    './samples/condor/boinc_gahp.exe',
    './samples/example_app/uc2.exe',
    './samples/example_app/ucn.exe',
    './samples/example_app/uc2_graphics.exe',
    './samples/example_app/slide_show.exe',
    './samples/multi_thread/multi_thread*.exe',
    './samples/sleeper/sleeper.exe',
    './samples/worker/worker*.exe',
    './samples/wrapper/wrapper*.exe',
    './samples/wrappture/wrappture_example.exe',
    './samples/wrappture/fermi.exe',
    './samples/sporadic/sporadic.exe',
    './samples/wsl_wrapper/wsl_wrapper*.exe',
    './samples/docker_wrapper/docker_wrapper*.exe',
]

android_apps_list = [
    # boinc_gahp
    './samples/condor/android_armv6_boinc_gahp',
    './samples/condor/android_arm_boinc_gahp',
    './samples/condor/android_arm64_boinc_gahp',
    './samples/condor/android_x86_boinc_gahp',
    './samples/condor/android_x86_64_boinc_gahp',
    # uc2
    './samples/example_app/android_armv6_uc2',
    './samples/example_app/android_arm_uc2',
    './samples/example_app/android_arm64_uc2',
    './samples/example_app/android_x86_uc2',
    './samples/example_app/android_x86_64_uc2',
    # ucn
    './samples/example_app/android_armv6_ucn',
    './samples/example_app/android_arm_ucn',
    './samples/example_app/android_arm64_ucn',
    './samples/example_app/android_x86_ucn',
    './samples/example_app/android_x86_64_ucn',
    # multi_thread
    './samples/multi_thread/android_armv6_multi_thread',
    './samples/multi_thread/android_arm_multi_thread',
    './samples/multi_thread/android_arm64_multi_thread',
    './samples/multi_thread/android_x86_multi_thread',
    './samples/multi_thread/android_x86_64_multi_thread',
    # sleeper
    './samples/sleeper/android_armv6_sleeper',
    './samples/sleeper/android_arm_sleeper',
    './samples/sleeper/android_arm64_sleeper',
    './samples/sleeper/android_x86_sleeper',
    './samples/sleeper/android_x86_64_sleeper',
    # worker
    './samples/worker/android_armv6_worker',
    './samples/worker/android_arm_worker',
    './samples/worker/android_arm64_worker',
    './samples/worker/android_x86_worker',
    './samples/worker/android_x86_64_worker',
    # wrapper
    './samples/wrapper/android_armv6_wrapper',
    './samples/wrapper/android_arm_wrapper',
    './samples/wrapper/android_arm64_wrapper',
    './samples/wrapper/android_x86_wrapper',
    './samples/wrapper/android_x86_64_wrapper',
    # wrappture_example
    './samples/wrappture/android_armv6_wrappture_example',
    './samples/wrappture/android_arm_wrappture_example',
    './samples/wrappture/android_arm64_wrappture_example',
    './samples/wrappture/android_x86_wrappture_example',
    './samples/wrappture/android_x86_64_wrappture_example',
    # fermi
    './samples/wrappture/android_armv6_fermi',
    './samples/wrappture/android_arm_fermi',
    './samples/wrappture/android_arm64_fermi',
    './samples/wrappture/android_x86_fermi',
    './samples/wrappture/android_x86_64_fermi',
    # sporadic
    './samples/sporadic/android_armv6_sporadic',
    './samples/sporadic/android_arm_sporadic',
    './samples/sporadic/android_arm64_sporadic',
    './samples/sporadic/android_x86_sporadic',
    './samples/sporadic/android_x86_64_sporadic'
]

windows_apps_list = [
    './win_build/Build/x64/Release/htmlgfx*.exe',
    './win_build/Build/x64/Release/wrapper*.exe',
    './win_build/Build/x64/Release/vboxwrapper*.exe',
    './win_build/Build/x64/Release/boincsim.exe',
    './win_build/Build/x64/Release/slide_show.exe',
    './win_build/Build/x64/Release/example*.exe',
    './win_build/Build/x64/Release/worker*.exe',
    './win_build/Build/x64/Release/sleeper*.exe',
    './win_build/Build/x64/Release/boinclog.exe',
    './win_build/Build/x64/Release/multi_thread*.exe',
    './win_build/Build/x64/Release/test*.exe',
    './win_build/Build/x64/Release/wrappture*.exe',
    './win_build/Build/x64/Release/crypt_prog.exe',
    './win_build/Build/x64/Release/wsl_wrapper*.exe',
    './win_build/Build/x64/Release/docker_wrapper*.exe',
    './win_build/Build/x64/Release/cudart*.dll',
    './win_build/Build/ARM64/Release/htmlgfx*.exe',
    './win_build/Build/ARM64/Release/wrapper*.exe',
    './win_build/Build/ARM64/Release/vboxwrapper*.exe',
    './win_build/Build/ARM64/Release/boincsim.exe',
    './win_build/Build/ARM64/Release/slide_show.exe',
    './win_build/Build/ARM64/Release/example*.exe',
    './win_build/Build/ARM64/Release/worker*.exe',
    './win_build/Build/ARM64/Release/sleeper*.exe',
    './win_build/Build/ARM64/Release/boinclog.exe',
    './win_build/Build/ARM64/Release/multi_thread*.exe',
    './win_build/Build/ARM64/Release/test*.exe',
    './win_build/Build/ARM64/Release/wrappture*.exe',
    './win_build/Build/ARM64/Release/crypt_prog.exe',
    './win_build/Build/ARM64/Release/wsl_wrapper*.exe',
    './win_build/Build/ARM64/Release/docker_wrapper*.exe',
]

windows_client_list = [
    './win_build/Build/x64/Release/boinc.exe',
    './win_build/Build/x64/Release/boincsvcctrl.exe',
    './win_build/Build/x64/Release/boinccmd.exe',
    './win_build/Build/x64/Release/boincscr.exe',
    './win_build/Build/x64/Release/boinc.scr',
    './win_build/Build/ARM64/Release/boinc.exe',
    './win_build/Build/ARM64/Release/boincsvcctrl.exe',
    './win_build/Build/ARM64/Release/boinccmd.exe',
    './win_build/Build/ARM64/Release/boincscr.exe',
    './win_build/Build/ARM64/Release/boinc.scr',
    './curl/ca-bundle.crt'
]

windows_manager_list = [
    './win_build/Build/x64/Release/boinctray.exe',
    './win_build/Build/x64/Release/boincmgr.exe',
    './win_build/Build/ARM64/Release/boinctray.exe',
    './win_build/Build/ARM64/Release/boincmgr.exe',
    './clientgui/skins',
    'locale/*/*.mo',
]

windows_installer_list = [
    './win_build/Build/x64/Release/boinccas.dll',
    './win_build/Build/ARM64/Release/boinccas.dll',
    './win_build/Build/x64/Release/installer_icon.exe',
    './win_build/Build/ARM64/Release/installer_icon.exe',
    './win_build/Build/x64/Release/installer.exe',
    './win_build/Build/ARM64/Release/installer.exe',
    './win_build/Build/x64/Release/boinc.msi',
    './win_build/Build/ARM64/Release/boinc.msi',
    './win_build/Build/x64/Release/installer_setup.exe',
    './win_build/Build/ARM64/Release/installer_setup.exe',
]

macos_manager_list = [
    'mac_build/build/Deployment/AddRemoveUser',
    'mac_build/build/Deployment/BOINC\ Installer.app',
    'mac_build/build/Deployment/BOINCManager.app',
    'mac_build/build/Deployment/BOINCSaver.saver',
    'mac_build/build/Deployment/BOINC_Finish_Install.app',
    'mac_build/build/Deployment/PostInstall.app',
    'mac_build/build/Deployment/SetUpSecurity',
    'mac_build/build/Deployment/SetVersion',
    'mac_build/build/Deployment/Uninstall\ BOINC.app',
    'mac_build/build/Deployment/boinc',
    'mac_build/build/Deployment/boinccmd',
    'mac_build/build/Deployment/boincscr',
    'mac_build/build/Deployment/gfx_cleanup',
    'mac_build/build/Deployment/gfx_switcher',
    'mac_build/build/Deployment/setprojectgrp',
    'mac_build/build/Deployment/switcher',
    'mac_build/build/Deployment/detect_rosetta_cpu',
    'mac_build/build/Deployment/Run_Podman',
    'mac_RemovePodman/build/Release/RemovePodman',
]

macos_apps_list = [
    'samples/docker_wrapper/build/Deployment/docker_wrapper',
    'samples/mac_build/build/Deployment/UC2-apple-darwin',
    'samples/mac_build/build/Deployment/UC2_graphics-apple-darwin',
    'samples/mac_build/build/Deployment/slide_show-apple-darwin',
    'samples/vboxwrapper/build/Deployment/vboxwrapper',
]

macos_makefile_apps_list = [
    'samples/wrapper/wrapper',
    'samples/vboxwrapper/vboxwrapper',
]

macos_apps_x86_64_list = [
    'samples/docker_wrapper/docker_wrapper_x86_64',
    'samples/example_app/x86_64/uc2',
    'samples/example_app/uc2_x86_64',
    'samples/example_app/x86_64/uc2_graphics',
    'samples/example_app/uc2_graphics_x86_64',
    'samples/example_app/x86_64/slide_show',
    'samples/example_app/slide_show_x86_64',
    'samples/openclapp/openclapp_x86_64',
]

macos_apps_arm64_list = [
    'samples/docker_wrapper/docker_wrapper_arm64',
    'samples/example_app/arm64/uc2',
    'samples/example_app/uc2_arm64',
    'samples/example_app/arm64/uc2_graphics',
    'samples/example_app/uc2_graphics_arm64',
    'samples/example_app/arm64/slide_show',
    'samples/example_app/slide_show_arm64',
    'samples/openclapp/openclapp_arm64',
]

logs_list = [
    'config.log',
    '3rdParty/linux/vcpkg/buildtrees/*.log',
    '3rdParty/osx/vcpkg/buildtrees/*.log',
    '3rdParty/android/vcpkg/buildtrees/*.log',
    '3rdParty/mingw/vcpkg/buildtrees/*.log',
    '3rdParty/Windows/vcpkg/buildtrees/*.log',
    'parts/boinc/build/3rdParty/linux/vcpkg/buildtrees/*.log',
    'android/BOINC/app/build/reports/',
    'mac_build/xcodebuild_*.log',
    'build/*.log',
]

def prepare_7z_archive(archive_name, target_directory, files_list):
    os.makedirs(target_directory, exist_ok=True)
    archive_path = os.path.join(target_directory, archive_name + '.7z')
    command = '7z a -t7z -r -mx=9 -xr!*.dSYM -xr!Makefile -xr!Makefile.* ' + archive_path + ' ' + " ".join(files_list)
    os.system(command)

def help():
    print('Usage: python preprare_deployment.py BOINC_TYPE')
    print('BOINC_TYPE : [' + " | ".join(boinc_types.keys()) + ']')

def prepare_linux_client(target_directory):
    prepare_7z_archive('linux_client', target_directory, linux_client_list)

def prepare_linux_client_vcpkg(target_directory):
    prepare_7z_archive('linux_client-vcpkg', target_directory, linux_client_list)

def prepare_linux_client_vcpkg_arm64(target_directory):
    prepare_7z_archive('linux_client-vcpkg-arm64', target_directory, linux_client_list)

def prepare_linux_client_vcpkg_armhf(target_directory):
    prepare_7z_archive('linux_client-vcpkg-armhf', target_directory, linux_client_list)

def prepare_linux_apps(target_directory):
    prepare_7z_archive('linux_apps', target_directory, linux_apps_list)

def prepare_linux_apps_arm64(target_directory):
    prepare_7z_archive('linux_apps-arm64', target_directory, linux_apps_list)

def prepare_linux_apps_armhf(target_directory):
    prepare_7z_archive('linux_apps-armhf', target_directory, linux_apps_list)

def prepare_linux_apps_vcpkg_arm64(target_directory):
    prepare_7z_archive('linux_apps-vcpkg-arm64', target_directory, linux_apps_list)

def prepare_linux_apps_vcpkg_armhf(target_directory):
    prepare_7z_archive('linux_apps-vcpkg-armhf', target_directory, linux_apps_list)

def prepare_linux_apps_vcpkg(target_directory):
    prepare_7z_archive('linux_apps-vcpkg', target_directory, linux_apps_list)

def prepare_linux_manager(target_directory):
    prepare_7z_archive('linux_manager', target_directory, linux_manager_list)

def prepare_linux_manager_with_webview(target_directory):
    prepare_7z_archive('linux_manager-with-webview', target_directory, linux_manager_list)

def prepare_linux_manager_with_webview_vcpkg(target_directory):
    prepare_7z_archive('linux_manager-with-webview-vcpkg', target_directory, linux_manager_list)

def prepare_linux_manager_with_webview_vcpkg_arm64(target_directory):
    prepare_7z_archive('linux_manager-with-webview-vcpkg-arm64', target_directory, linux_manager_list)

def prepare_linux_manager_with_webview_vcpkg_armhf(target_directory):
    prepare_7z_archive('linux_manager-with-webview-vcpkg-armhf', target_directory, linux_manager_list)

def prepare_linux_manager_without_webview(target_directory):
    prepare_7z_archive('linux_manager-without-webview', target_directory, linux_manager_list)

def prepare_win_apps_mingw(target_directory):
    prepare_7z_archive('win_apps-mingw', target_directory, mingw_apps_list)

def prepare_win_apps_mingw_vcpkg(target_directory):
    prepare_7z_archive('win_apps-mingw-vcpkg', target_directory, mingw_apps_vcpkg_list)

def prepare_android_apps(target_directory):
    prepare_7z_archive('android_apps', target_directory, android_apps_list)

def prepare_win_apps(target_directory):
    prepare_7z_archive('win_apps', target_directory, windows_apps_list)

def prepare_win_client(target_directory):
    prepare_7z_archive('win_client', target_directory, windows_client_list)

def prepare_win_manager(target_directory):
    prepare_7z_archive('win_manager', target_directory, windows_manager_list)

def prepare_win_installer(target_directory):
    prepare_7z_archive('win_installer', target_directory, windows_installer_list)

def prepare_macos_apps(target_directory):
    prepare_7z_archive('macos_manager', target_directory, macos_manager_list)
    prepare_7z_archive('macos_apps', target_directory, macos_apps_list)

def prepare_macos_makefile_apps(target_directory):
    prepare_7z_archive('macos_apps', target_directory, macos_makefile_apps_list)
    prepare_7z_archive('macos_apps_x86_64', target_directory, macos_apps_x86_64_list)
    prepare_7z_archive('macos_apps_arm64', target_directory, macos_apps_arm64_list)

def prepare_logs(target_directory):
    prepare_7z_archive('logs', target_directory, logs_list)

boinc_types = {
    'android_apps': prepare_android_apps,
    'linux_apps': prepare_linux_apps,
    'linux_apps-arm64': prepare_linux_apps_arm64,
    'linux_apps-armhf': prepare_linux_apps_armhf,
    'linux_apps-vcpkg': prepare_linux_apps_vcpkg,
    'linux_apps-vcpkg-arm64': prepare_linux_apps_vcpkg_arm64,
    'linux_apps-vcpkg-armhf': prepare_linux_apps_vcpkg_armhf,
    'linux_client': prepare_linux_client,
    'linux_client-vcpkg': prepare_linux_client_vcpkg,
    'linux_client-vcpkg-arm64': prepare_linux_client_vcpkg_arm64,
    'linux_client-vcpkg-armhf': prepare_linux_client_vcpkg_armhf,
    'linux_manager': prepare_linux_manager,
    'linux_manager-with-webview': prepare_linux_manager_with_webview,
    'linux_manager-with-webview-vcpkg': prepare_linux_manager_with_webview_vcpkg,
    'linux_manager-with-webview-vcpkg-arm64': prepare_linux_manager_with_webview_vcpkg_arm64,
    'linux_manager-with-webview-vcpkg-armhf': prepare_linux_manager_with_webview_vcpkg_armhf,
    'linux_manager-without-webview': prepare_linux_manager_without_webview,
    'logs': prepare_logs,
    'macos_manager': prepare_macos_apps,
    'macos_samples-makefile': prepare_macos_makefile_apps,
    'win_apps': prepare_win_apps,
    'win_apps-mingw': prepare_win_apps_mingw,
    'win_apps-mingw-vcpkg': prepare_win_apps_mingw_vcpkg,
    'win_client': prepare_win_client,
    'win_installer': prepare_win_installer,
    'win_manager': prepare_win_manager,
}

if (len(sys.argv) != 2):
    help()
    sys.exit(1)

boinc_type = sys.argv[1]
target_dir = 'deploy'

if (boinc_type not in boinc_types):
    print('Unknown BOINC_TYPE: ' + boinc_type)
    help()
    sys.exit(1)

boinc_types[boinc_type](target_dir)



#!/usr/bin/env python

## $Id$

# DOESN'T WORK YET

from test_uc import *

class ProjectSchedMove(ProjectUC):
    def run(self):
        self.sched_install('feeder')
        self.start_servers()


if __name__ == '__main__':
    test_msg("backing off and finding new scheduler URL from master URL")
    verbose_echo(0, "BEFORE SCHEDULER MOVE")
    proj = ProjectSchedMove(short_name='test_sched_moved')
    proj.install()
    run_check_all()

    verbose_echo(0, "AFTER SCHEDULER MOVE")
    # TODO: change scheduler url
    proj.install_project()
    proj.install_works()
    run_check_all()




#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib
import sys

def help():
    print('Usage:')
    print('python source-code-check.py <dir>')

def check(directory, bytes_to_check, exclude_dirs, exclude_extensions, exclude_files):
    if not os.path.isdir(directory):
        return None

    files_with_errors = []

    for root, dirs, files in os.walk(directory, topdown=True):
        dirs[:] = [d for d in dirs if os.path.join(root, d) not in exclude_dirs]
        for filename in files:
            if (pathlib.Path(filename).suffix in exclude_extensions):
                continue
            file = os.path.join(root, filename)
            if (os.path.islink(file) or file in exclude_files):
                continue
            with open(file, "rb") as f:
                print('Checking file: ' + file)
                byte = f.read(1)
                while byte:
                    if byte in bytes_to_check:
                        files_with_errors.append(file)
                        break
                    byte = f.read(1)
    return files_with_errors


if (len(sys.argv) != 2):
    help()
    sys.exit(1)

directory = sys.argv[1]

exclude_dirs = [
    os.path.join(directory, ".git"),
    os.path.join(directory, "3rdParty", "android"),
    os.path.join(directory, "3rdParty", "buildCache"),
    os.path.join(directory, "3rdParty", "linux"),
    os.path.join(directory, "3rdParty", "Windows"),
    os.path.join(directory, "3rdParty", "vcpkg_ports", "ports"),
    os.path.join(directory, "android", "BOINC", ".gradle"),
    os.path.join(directory, "android", "BOINC", "app", "build"),
    os.path.join(directory, "android", "BOINC", "app", "src", "main", "assets"),
    os.path.join(directory, "android", "BOINC", "app", "src", "main", "res"),
    os.path.join(directory, "doc", "manpages"),
    os.path.join(directory, "fastlane"),
    os.path.join(directory, "samples", "example_app", "bin"),
]

exclude_files = [
    os.path.join(directory, "aclocal.m4"),
    os.path.join(directory, "android", "BOINC", "gradlew"),
    os.path.join(directory, "client", "boinc_client"),
    os.path.join(directory, "client", "boinccmd"),
    os.path.join(directory, "client", "boinc"),
    os.path.join(directory, "client", "switcher"),
    os.path.join(directory, "clientgui", "BOINCBaseView.cpp"),
    os.path.join(directory, "clientscr", "progress", "simt"),
    os.path.join(directory, "html", "inc", "GeoIP.dat"),
    os.path.join(directory, "mac_installer", "BOINC.pmproj"),
    os.path.join(directory, "mac_build", "boinc.xcodeproj", "project.pbxproj"),
    os.path.join(directory, "stage", "usr", "local", "bin", "boinc_client"),
    os.path.join(directory, "stage", "usr", "local", "bin", "boinccmd"),
    os.path.join(directory, "stage", "usr", "local", "bin", "boinc"),
    os.path.join(directory, "stage", "usr", "local", "bin", "switcher"),
]

exclude_extensions = [
    ".a",
    ".bmp",
    ".cub",
    ".dll",
    ".exe",
    ".gif",
    ".icns",
    ".ico",
    ".jar",
    ".jpg",
    ".lib",
    ".mo",
    ".nib",
    ".o",
    ".odp",
    ".pdf",
    ".pdn",
    ".png",
    ".po",
    ".psd",
    ".rgb",
    ".so",
    ".tif",
    ".tiff",
    ".tlb",
    ".ttf",
    ".zip",
]

files_with_errors = check(directory, [b"\xC2"], exclude_dirs, exclude_extensions, exclude_files)

if files_with_errors:
    print("Found files with errors:")
    for file in files_with_errors:
        print(file)
    sys.exit(1)

sys.exit(0)



#!/usr/bin/env python

## $Id$

from test_uc import *

if __name__ == '__main__':
    test_msg("scheduler exponential backoff (master url failure)")
    Proxy('close_connection if $nconnections < 2; ', html=1)
    # Proxy(('print "hi nc=$nconnections start=$start nchars=$nchars url=$url\n"; ' +
    #        'close_connection if $nconnections < 2; '),
    #       html=1)
    # Proxy(( 'print "hi nc=$nconnections start=$start nchars=$nchars url=$url\n"; ' +
    #        ''),
    #       html=1)
    ProjectUC(short_name='test_masterurl_failure')
    run_check_all()

    ## TODO: verify it took ??? seconds



#!/usr/bin/env python

## $Id$

from testbase import *
import urllib, random

# Test makes sure that testing framework is sane:
#
# - executables compiled
# - cgi server works
# - test proxy works
# - mysql permissions and command-line client works

def read_url(url, quiet=False):
    '''return 1 line from url'''
    verbose_echo(2, "   reading url: "+url)
    err = ''
    try:
        return urllib.URLopener().open(url).readline().strip()
    except IOError, e:
        err = e
    except AttributeError:
        # Python urllib is buggy if connection is closed (by our proxy
        # intentionally) right after opened
        pass
    if not quiet:
        error("couldn't access url: %s %s" % (url, err))
    else:
        verbose_echo(2, "couldn't access url: %s %s" % (url, err))
    return ''

if __name__ == '__main__':
    test_msg("framework sanity")

    # verbose_echo(1, "Checking executables")
    # check_core_client_executable()
    # check_app_executable("upper_case")
    # check_app_executable("concat")
    # check_app_executable("1sec")

    verbose_echo(1, "Checking directories")
    for d in ['projects_dir',
              'cgi_dir', 'html_dir', 'hosts_dir']:
        dir = options.__dict__[d]
        if not os.path.isdir(dir):
            error("%s doesn't exist: %s" % (d, dir))

    magic = "Foo %x Bar" % random.randint(0,2**16)

    html_path = os.path.join(options.html_dir, 'test_sanity.txt')
    html_url  = os.path.join(options.html_url, 'test_sanity.txt')
    html_proxy_url = proxerize(html_url)
    cgi_path  = os.path.join(options.cgi_dir,  'test_sanity_cgi')
    cgi_url   = os.path.join(options.cgi_url,  'test_sanity_cgi')

    verbose_echo(1, "Checking webserver setup: non-cgi")
    print >>open(html_path,'w'), magic
    if read_url(html_url) != magic:
        error("couldn't access a file I just wrote: "+html_path+"\n  using url: "+html_url)

    verbose_echo(1, "Checking proxy setup")
    if read_url(html_proxy_url, quiet=True):
        error("Another proxy already running")
    else:
        proxy = Proxy('')
        if read_url(html_proxy_url) != magic:
            error("couldn't access file using proxy url: "+html_proxy_url)
        else:
            proxy.stop()

            proxy = Proxy('close_connection if $nconnections < 2')
            if read_url(html_proxy_url, quiet=True):
                error("Proxy should have closed connection #1")
                if read_url(html_proxy_url) != magic:
                    error("Proxy should have allowed connection #2")
        proxy.stop()

    os.unlink(html_path)

    verbose_echo(1, "Checking webserver setup: cgi")
    print >>open(cgi_path,'w'), '''#!/bin/sh
echo "Content-Type: text/plain"
echo ""
echo "%s"
''' % magic
    os.chmod(cgi_path, 0755)

    if read_url(cgi_url) != magic:
        error("couldn't access a cgi file I just wrote: "+cgi_path+"\n  using url: "+cgi_url)

    os.unlink(cgi_path)


    database_name = 'boinc_test_sanity_mysql_%s_%d'%(
        os.environ['USER'], random.randint(0,2**16))

    # create and drop a database
    verbose_echo(1, "Checking mysql commandline and permissions")
    shell_call('echo "create database %s" | mysql' % database_name)
    shell_call('echo "drop database %s" | mysql' % database_name)



# $Id$

# boinc_project_path.py - defines the directories where config.xml and
# run_state.xml are kept.
#
# You can override these defaults with one of these options:
#    1) modify this file directly (if you have only one project on your server
#       or have separate copies for each)
#    2) create a new boinc_project_path.py and place it earlier in PYTHONPATH
#       than the default one
#    3) define environment variables

import sys, os, socket

# Default locations to look for config.xml: ".", "..", "../..", and the parent
# directory of the script's location
_dirs = [ os.curdir, os.pardir, os.path.join(os.pardir, os.pardir),
          os.path.join(os.path.dirname(sys.argv[0]), os.pardir) ]

# BOINC_PROJECT_DIR takes precedence if it is defined
_project_dir = ''
if os.environ.get('BOINC_PROJECT_DIR'):
    _project_dir = os.environ.get('BOINC_PROJECT_DIR')
else:
    for dir in _dirs:
        if os.path.exists(os.path.join(dir, 'config.xml')):
            _project_dir = os.path.abspath(dir)
            break

_local_hostname = socket.gethostname()
_local_hostname = _local_hostname.split('.')[0]

def project_path(name = None):
    if name:
        return os.path.join(_project_dir, name)
    else:
        return _project_dir

config_xml_filename = project_path('config.xml')
project_xml_filename = project_path('project.xml')
run_state_xml_filename = project_path('run_state_' + _local_hostname + '.xml')



#!/usr/bin/env python3

import boinc_path_config
from Boinc.assimilator import *

class TestAssimilator(Assimilator):
    """
    A minimal Assimilator example that dumps the result out to the log
    """

    def __init__(self):
        Assimilator.__init__(self)

    def assimilate_handler(self, wu, results, canonical_result):
        """
        Assimilates a canonical result, in this case assimilation
        means dumping the contents of the result to the log.
        Also calls report_errors to log any problems present in the workunit (wu)
        """

        # check for valid wu.canonical_result
        if wu.canonical_result:
            # do application specific processing
            self.logNormal("[%s] Found canonical result\n", wu.name)
            result = self.get_file_path(canonical_result)
            for line in open(result, 'r').readlines():
                line = line.strip()
                self.logDebug("  [%s] Answer found %s %s\n", canonical_result.name, line[-32:], line[:-33])
        else:
            self.logNormal("[%s] No canonical result\n", wu.name)

        if self.report_errors(wu):
            # report_errors returns true if error state was present
            # perhaps add some special logic here
            # even if no logic is required, report_errors should be called
            pass

# allow the module to be executed as an application
if __name__ == '__main__':
    asm = TestAssimilator()
    asm.run()



#!/usr/bin/env python

## $Id$

# tests whether command-line arg passing works

from testbase import *

class WorkConcat(Work):
    def __init__(self, redundancy, **kwargs):
        Work.__init__(self, redundancy=redundancy)
        self.wu_template = "concat_wu"
        self.result_template = "concat_result"
        self.input_files = ['input']*2
        self.__dict__.update(kwargs)

class ProjectConcat(TestProject):
    def __init__(self, works=None, users=None, hosts=None):
        (num_wu, redundancy) = get_redundancy_args()
        TestProject.__init__(self,
                             appname = 'concat',
                             num_wu=num_wu, redundancy=redundancy,
                             expected_result = Result(),
                             works = works or [WorkConcat(redundancy=redundancy)],
                             users = users,
                             hosts = hosts)

    # def check(self):
    #     self.sched_run('validate_test')
    #     result = {}
    #     result['server_state'] = RESULT_SERVER_STATE_OVER
    #     self.check_results(result)
    #     self.check_files_match("upload/concat_wu_%d_0", "concat_correct_output", count=self.redundancy)
    #     self.sched_run('assimilator')
    #     self.sched_run('file_deleter')
    #     self.check_deleted("download/input")
    #     self.check_deleted("upload/concat_wu_%d_0", count=self.redundancy)

    # def run(self):
    #     self.install()
    #     self.sched_install('feeder')
    #     self.start_servers()

if __name__ == '__main__':
    test_msg("standard concat application");
    ProjectConcat()
    run_check_all()



## $Id$

from __future__ import print_function
from Boinc import configxml

try:
    # use new hashlib if available
    from hashlib import md5
except:
    import md5
import os, shutil, binascii, filecmp

# from http://www.plope.com/software/uuidgen/view
_urandomfd = None
def urandom(n):
    """urandom(n) -> str

    Return a string of n random bytes suitable for cryptographic use.

    """
    global _urandomfd
    if _urandomfd is None:
        try:
            _urandomfd = os.open("/dev/urandom", os.O_RDONLY)
        except:
            _urandomfd = NotImplementedError
    if _urandomfd is NotImplementedError:
        raise NotImplementedError("/dev/urandom (or equivalent) not found")
    bytes = ""
    while len(bytes) < n:
        bytes += os.read(_urandomfd, n - len(bytes))
    return bytes

def make_uuid():
    return binascii.hexlify(urandom(16))

def md5_file(path):
    """
    Return a 16-digit MD5 hex digest of a file's contents
    Read the file in chunks
    """

    chunk = 8096

    try:
        checksum = md5()
    except TypeError:
        checksum = md5.new()

    fp = open(path, 'r')
    while True:
        buffer = fp.read(chunk)
        if not buffer:
            break
        checksum.update(buffer)

    fp.close()

    return checksum

def file_size(path):
    """Return the size of a file"""
    f = open(path)
    f.seek(0,2)
    return f.tell()

# workaround for dubious function change in python3
#
from sys import version_info
def input_aux():
    if version_info.major == 2:
        return raw_input()
    return input()

def query_yesno(str):
    '''Query user; default Yes'''
    print (str, "[Y/n] ", end="")
    return not input_aux().strip().lower().startswith('n')

def query_noyes(str):
    '''Query user; default No'''
    print (str, "[y/N] ", end="")
    return input_aux().strip().lower().startswith('y')

def get_output_file_path(filename):
    """ Return the filename's path in the upload directory
        Use this if you're developing a validator/assimilator in Python
    """
    config = configxml.default_config()
    fanout = long(config.config.uldl_dir_fanout)
    try:
        s = md5(filename).hexdigest()[1:8]
    except TypeError:
        s = md5.new(filename).hexdigest()[1:8]
    x = long(s, 16)
    return "%s/%x/%s" % (config.config.upload_dir, x % fanout, filename)



#!/usr/bin/env python

# $Id$

# fake php - all we really need is 'include schedulers.txt'

import os, sys

REQUEST_URI = os.environ['REQUEST_URI']

print 'Content-Type: text/plain'
print

print """--- FAKE PHP ---

[ REQUEST_URI=%s ]

Since I can't find php4 on your system, this stub program fake_php.py just
prints schedulers.txt as necessary.

"""%REQUEST_URI

if REQUEST_URI.endswith('/index.php'):
    sys.stdout.write(open('schedulers.txt').read())



#!/usr/bin/env python

import boinc_path_config
from assimilator import *
import os, sys, shutil

class PymwAssimilator(Assimilator):
    """
    PyMW Assimilator. Copies workunit results to a predefined output directory.
    """

    def __init__(self):
        Assimilator.__init__(self)
        self.pymwDir = None

    def _copy_to_output(self, result, error_mask=0):
        # validate that the destination path still exists
        if not os.path.exists(self.pymwDir):
            self.logCritical("PyMW path does not exist or is inaccessible: %s\n", \
                            self.pymwDir)
            return

        resultFullPath = self.get_file_path(result)
        resultName = re.search('<open_name>(.*)</open_name>',result.xml_doc_in).group(1)

        # validate that the source path is accessible
        if not error_mask and not os.path.exists(resultFullPath):
            self.logCritical("Result path does not exist or is inaccessible: %s\n", \
                            resultFullPath)
            return

        # copy the file to the output directory where it
        # will be processed by PyMW
        try:
            dest = os.path.join(self.pymwDir, resultName)
            if error_mask:
                dest += ".error"
                f = open(dest, "w")
                try:
                    f.writelines("BOINC error: " + str(error_mask) + "\n")
                    if result.stderr_out:
                        f.writelines("STD ERR: " + result.stderr_out + "\n")
                    f.writelines("For additional information, check: " +
                                 "$project/log_$machine/pymw_assimilator.py.log\n")
                finally: f.close()
                self.logNormal("Error flag created [%s]\n", resultName)
            else:
                shutil.copy2(resultFullPath, dest)
                self.logNormal("Result copied [%s]\n", resultName)
        except (Exception, msg):
            self.logCritical("Error copying output\n" + \
                             "  - Source: %s\n" + \
                             "  - Dest: %s\n" +
                             "  - Error: %s",
                             resultFullPath, dest, msg)

    def assimilate_handler(self, wu, results, canonical_result):
        """
        Assimilates a canonical result by copying the result file
        to the PyMW pickup directory, self.pymwDir
        """

        # check for valid wu.canonical_result
        if wu.canonical_result:
            self.logNormal("[%s] Found canonical result\n", wu.name)
            self._copy_to_output(canonical_result, wu.error_mask)
        elif wu.error_mask != 0:
            # this is an error
            self.logNormal("[%s] Workunit failed, sending arbitrary result\n", wu.name)
            self._copy_to_output(results[0], wu.error_mask)
            self.logNormal("[%s] No canonical result\n", wu.name)
        else:
            self.logNormal("[%s] No canonical result\n", wu.name)

        # report errors with the workunit
        if self.report_errors(wu):
            pass


    def parse_args(self, args):
        """
        This overridden version adds support for a PyMW destination directory.
        """

        # scan the args for the -pymw_dir switch
        # remove it when found so the base class doesn't complain
        # then call the base parse_args method

        newArgs = []
        args.reverse()
        found = False

        while len(args):
            arg = args.pop()
            if arg.strip() == '-pymw_dir':
                if len(args) == 0:
                    self.logCritical("Error, path switch found, but no path specified\n")
                    sys.exit(1)

                self.pymwDir = args.pop()
                if not os.path.exists(self.pymwDir):
                    self.logNormal("Warning, path does not exist or is inaccessible: %s\n", \
                                    self.pymwDir)
                    found = True
                else:
                    found = True
            else:
                newArgs.append(arg)

        if not found:
            self.logCritical("Error, path argument expected: -pymw_dir <PATH>\n")
            sys.exit(1)

        Assimilator.parse_args(self, newArgs)

# allow the module to be executed as an application
if __name__ == '__main__':
    asm = PymwAssimilator()
    asm.run()



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import os
import pathlib
import re
import sys

def help():
    print('Usage:')
    print('python update_copyright_year.py <dir>')

def update_dlgabout(filename, year=datetime.date.today().year):
    print('Checking file: ' + filename)
    with open(filename, "rb") as f:
        lines = f.read().splitlines()
    count = 0
    changed = False
    match_found = False
    for line in lines:
        count += 1
        match = re.search(b'.*\(C\)\s\d{4}-\d{4}.*', line)
        if match:
            match_found = True
            updated_line = re.sub(b'-(\d{4})', b'-' + str.encode(str(year)), line)
            if updated_line != line:
                lines[count-1] = updated_line
                changed = True
    if not match_found:
        print('There was no copyright line found in the file')
        return False
    if changed:
        print('Updating file: ' + filename + '...')
        with open(filename, "wb") as f:
            f.write(str.encode(os.linesep).join(lines))
            f.write(str.encode(os.linesep))
    return True

def update_rc_files(directory, year=datetime.date.today().year):
    if not os.path.isdir(directory):
        return None
    dlg_about = os.path.join(directory, "clientgui", "DlgAbout.cpp")
    for root, dirs, files in os.walk(directory, topdown=True):
        dirs[:] = [d for d in dirs if os.path.join(root, d) not in exclude_dirs]
        for filename in files:
            src_file = os.path.join(root, filename)
            if src_file == dlg_about:
                if not update_dlgabout(os.path.join(root, filename)):
                    return False
                continue
            if (pathlib.Path(filename).suffix != '.rc'):
                continue
            if (os.path.islink(src_file)):
                continue
            with open(src_file, "rb") as f:
                print('Checking file: ' + src_file)
                lines = f.read().splitlines()
            count = 0
            changed = False
            for line in lines:
                count += 1
                match = re.search(b'VALUE \"LegalCopyright\", \".*\d{4}-\d{4}.*\"', line)
                if match:
                    updated_line = re.sub(b'-(\d{4})', b'-' + str.encode(str(year)), line)
                    if updated_line != line:
                        lines[count-1] = updated_line
                        changed = True
            if changed:
                print('Updating file: ' + src_file + '...')
                with open(src_file, "wb") as f:
                    f.write(str.encode(os.linesep).join(lines))
                    f.write(str.encode(os.linesep))
    return True


if (len(sys.argv) < 2):
    help()
    sys.exit(1)

directory = sys.argv[1]

exclude_dirs = [
    os.path.join(directory, "3rdParty")
]

if not update_rc_files(directory):
    sys.exit(1)

sys.exit(0)



# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2024 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import re

class TestHelper:
    def get_current_version_number(self):
        with open("version.h", "r") as f:
            lines = f.readlines()
            for line in lines:
                res = re.search("#define BOINC_VERSION_STRING \"([\d]+.[\d]+.[\d]+)\"", line)
                if res is not None:
                    return res[1]
        return ""

    def get_version_from_string(self, string):
        res = re.search("([\d]+.[\d]+.[\d]+)", string)
        if res is not None:
            return res[1]
        return ""



# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2024 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

class TestSet:
    def __init__(self, name):
        self.name = name
        self.passed = 0
        self.failed = 0
        print(("Running test set [{name}]...").format(name=self.name))

    def __del__(self):
        print(("Test set [{name}]: \033[92mpassed [{passed}]\033[0m, \033[91mfailed [{failed}]\033[0m.").format(name=self.name, passed=self.passed, failed=self.failed))

    def _print_success_report(self, test_name):
        self.passed += 1
        self._print_success(("Passed [{test_name}]").format(test_name=test_name))

    def _print_success(self, message):
        print(("\033[92m{message}\033[0m").format(message=message))

    def _print_failure(self, message):
        print(("\033[91m{message}\033[0m").format(message=message))

    def result(self):
        return self.failed == 0

    def expect_true(self, condition, test_name):
        if not condition:
            self.failed += 1
            self._print_failure(("Failed [{test_name}]: expected True, got False").format(test_name=test_name))
            return False
        self._print_success_report(test_name)
        return True

    def expect_false(self, condition, test_name):
        if condition:
            self.failed += 1
            self._print_failure(("Failed [{test_name}]: expected False, got True").format(test_name=test_name))
            return False
        self._print_success_report(test_name)
        return True

    def expect_equal(self, expected, actual, test_name):
        if expected != actual:
            self.failed += 1
            self._print_failure(("Failed [{test_name}]: expected [{expected}], got [{actual}]").format(test_name=test_name, expected=expected, actual=actual))
            return False
        self._print_success_report(test_name)
        return True

    def expect_not_equal(self, a, b, test_name):
        if a == b:
            self.failed += 1
            self._print_failure(("Failed [{test_name}]: expected {a} != {b}").format(test_name=test_name, a=a, b=b))
            return False
        self._print_success_report(test_name)



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import sys

def set_configure_ac(version):
    with open('configure.ac', 'r') as f:
        lines = f.readlines()
    with open('configure.ac', 'w') as f:
        for line in lines:
            if line.startswith('WORKER_RELEASE='):
                line = f'WORKER_RELEASE={version}\n'
            f.write(line)

def set_version_h(version):
    with open('version.h', 'r') as f:
        lines = f.readlines()
    with open('version.h', 'w') as f:
        for line in lines:
            if line.startswith('#define WORKER_RELEASE'):
                line = f'#define WORKER_RELEASE {version}\n'
            f.write(line)

def set_vcxproj(version):
    for vcxproj in ['win_build/worker.vcxproj']:
        with open(vcxproj, 'r') as f:
            lines = f.readlines()
        with open(vcxproj, 'w') as f:
            for line in lines:
                if line.startswith('    <TargetVersion>'):
                    line = f'    <TargetVersion>{version}</TargetVersion>\n'
                f.write(line)

if (len(sys.argv) != 2):
    print('Usage: set-worker-version.py VERSION')
    exit(1)

version = sys.argv[1]

print(f'Setting worker version to {version}...')

set_configure_ac(version)
set_version_h(version)
set_vcxproj(version)

print('Done.')



#!/usr/bin/env python

# boincxml.py - XML utilities for boinc

from __future__ import print_function
import sys, os
import xml.dom.minidom

def append_new_element(parent_node, name):
    new_element = xml.dom.minidom.Element(name)
    if isinstance(parent_node,xml.dom.minidom.Document):
        new_element.ownerDocument = parent_node
    else:
        assert(parent_node.ownerDocument)
        new_element.ownerDocument = parent_node.ownerDocument
    parent_node.appendChild(new_element)
    return new_element

def get_elements(node, name):
    return node.getElementsByTagName(name)

def get_element(node, name, optional=True):
    try:
        return get_elements(node,name)[0]
    except IndexError:
        if optional:
            return append_new_element(node, name)
        raise SystemExit("ERROR: Couldn't find xml node <%s>"% name)

def _None2Str(object):
    if object is None:
        return ''
    else:
        return object

def get_element_data(node):
    return node and _None2Str(node.firstChild and node.firstChild.data)

def get_element_int(node, default=0):
    try:
        return int(get_element_data(node))
    except:
        return default

def get_child_elements(node):
    return filter(lambda node: node.nodeType == node.ELEMENT_NODE, node.childNodes)

def set_element(node, new_data):
    if node.firstChild and node.firstChild.data:
        node.firstChild.data = str(new_data)
    else:
        # different versions of pyxml have different interface (!)
        try:
            new_data_node = xml.dom.minidom.Text()
        except:
            new_data_node = xml.dom.minidom.Text('')
        new_data_node.data = str(new_data)
        new_data_node.ownerDocument = node.ownerDocument
        node.appendChild(new_data_node)

def strip_white_space(node):
    ''' strip white space from text nodes, and remove empty nodes. '''
    # the [:] below is to copy the list since removing children modifies the
    # list.
    for child in node.childNodes[:]:
        if child.nodeType == child.TEXT_NODE:
            child.data = child.data.strip()
            if not child.data:# and (child.nextSibling or child.previousSibling):
                node.removeChild(child)
        else:
            strip_white_space(child)

def get_elements_as_dict(node):
    dict = {}
    for child in get_child_elements(node):
        # note: str() changes from unicode to single-byte strings
        dict[str(child.nodeName)] = get_element_data(child)
    return dict

class ConfigDict:
    def __init__(self, dom_node):
        self._node = dom_node
        self._name = self._node.nodeName
        self.__dict__.update(get_elements_as_dict(self._node))
    def save(self):
        for key in self.__dict__:
            if key.startswith('_'): continue
            set_element( get_element(self._node,key,1), str(self.__dict__[key]) )
    def debug_print(self):
        for key in self.__dict__.keys():
            print(key.rjust(15), '=', self.__dict__[key])

class ConfigDictList(list):
    def __init__(self, dom_node, item_class=ConfigDict):
        self._node = dom_node
        list.__init__(self, map(item_class, get_child_elements(self._node)))
    def save(self):
        for item in self:
            ConfigDict.save(item)
    def make_node_and_append(self, name):
        '''Make a new ConfigDict and append it. Returns new ConfigDict.'''
        new_element = append_new_element(self._node, name)
        new_cd      = ConfigDict(new_element)
        self.append(new_cd)
        return new_cd
    def remove_node(self, item):
        self.remove(item)
        self._node.removeChild(item._node)

class XMLConfig:
    '''Base class for xml config files'''
    def __init__(self, filename = None):
        # default_filename should be defined by children
        self.filename = filename or self.default_filename
    def _init_empty_xml(self):
        self.xml = xml.dom.minidom.parseString(self.default_xml)
    def init_empty(self):
        self._init_empty_xml()
        self._get_elements()
        return self
    def read(self, failopen_ok=False):
        """Read the XML object's file source and return self."""
        if failopen_ok and not os.path.exists(self.filename):
            self._init_empty_xml()
        else:
            try:
                self.xml = xml.dom.minidom.parse(self.filename)
                strip_white_space(self.xml)
            except:
                print("Warning: couldn't parse XML file:", self.filename, file=sys.stderr)
                if not failopen_ok:
                    raise Exception("Couldn't parse XML file\n")
                self._init_empty_xml()
        try:
            self._get_elements()
        except:
            if not failopen_ok:
                raise Exception("%s: Couldn't get elements from XML file")
        return self
    def _get_elements(self):
        pass
    def write(self, output=None):
        """Write XML data to filename, or other stream."""
        self._set_elements()
        if not output:
            output = open(self.filename,'w')
        self.xml.writexml(output, "", " "*4, "\n")
        output.write('')
        return self
    def _set_elements(self):
        pass



#! /usr/bin/env python3

# check that a file is uppercase

import sys

def is_uc(path):
    with open(path) as f:
        data = f.read()
    return data == data.upper()

exit(0 if is_uc(sys.argv[1]) else 1)



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import sys
import zipfile

def rename_file(directory, old_name, new_name):
    os.rename(f"{directory}/{old_name}", f"{directory}/{new_name}")
    print(f"Renamed {old_name} to {new_name}")

def extract_zip(zip_path, extract_to):
    with zipfile.ZipFile(zip_path, 'r') as zip_ref:
        zip_ref.extractall(extract_to)
    print(f"Extracted {zip_path} to {extract_to}")

def extract_7z(archive_path, target_file, extract_to, sevenzip_path):
    command = [sevenzip_path, 'e', archive_path, f'-o{extract_to}', target_file]
    subprocess.run(command, check=True)
    print(f"Extracted {archive_path} to {extract_to}")

def rename_pdb(directory, filename):
    arch = filename.split('_')[2]
    arch = 'x86_64' if arch == 'x64' else arch
    rename_file(directory, filename, f"boinc_{version}_windows_{arch}.pdb.zip")

def rename_installer(directory, filename, sevenzip_path):
    arch = filename.split('_')[2]
    arch = 'x86_64' if arch == 'x64' else arch
    target_file = 'installer_setup.exe'
    extract_zip(f"{directory}/{filename}", directory)
    extract_7z(f"{directory}/win_installer.7z", target_file, directory, sevenzip_path)
    rename_file(directory, target_file, f"boinc_{version}_windows_{arch}.exe")

def rename_linux_package(directory, filename):
    codename = filename.split('_')[3]
    if codename.startswith('suse'):
        codename = codename + "_" + filename.split('_')[4]
    package_name = zipfile.ZipFile(f"{directory}/{filename}").filelist[0].filename
    extract_zip(f"{directory}/{filename}", directory)
    new_package_name = package_name.replace(".deb", f"_{codename}.deb") if package_name.endswith(".deb") else package_name.replace(".rpm", f"_{codename}.rpm")
    rename_file(directory, package_name, new_package_name)

def rename_android(directory, filename):
    apk_name = zipfile.ZipFile(f"{directory}/{filename}").filelist[0].filename
    extract_zip(f"{directory}/{filename}", directory)
    arch = "" if filename.split('_')[1] == "manager" else f"_{filename.split('_')[1]}"
    rename_file(directory, apk_name, f"boinc_{version}{arch}.apk")

if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("Usage: prepare_release_artifacts.py <directory> <version> <7z_path>")
        sys.exit(1)

    directory = sys.argv[1]
    version = sys.argv[2]
    sevenzip_path = sys.argv[3]

    print(f"Preparing release artifacts for version {version} in directory {directory}...")

    files = os.listdir(directory)
    for filename in files:
        file_path = os.path.join(directory, filename)
        if os.path.isfile(file_path):
            print(f"Processing file: {filename}")
            if filename.startswith("win_pdb_"):
                rename_pdb(directory, filename)
            elif filename.startswith("win_installer_"):
                rename_installer(directory, filename, sevenzip_path)
                os.remove(f"{directory}/win_installer.7z")
                os.remove(file_path)
            elif filename.startswith("linux-package_"):
                rename_linux_package(directory, filename)
                os.remove(file_path)
            elif filename.startswith("android_"):
                rename_android(directory, filename)
                os.remove(file_path)
            else:
                print(f"Skipping unrecognized file: {filename}")

    print("Release artifacts prepared successfully.")



#!/usr/bin/env python

# add_util.py - used in make_project

from __future__ import print_function
from Boinc import database, tools
import time, pprint
import MySQLdb

CREATE_TIME = ['?create_time', int(time.time())]
TRANSITION_TIME = ['?transition_time', int(time.time())]

class XAppVersion(database.AppVersion):
    def __init__(self,**kwargs):
        kwargs['xml_doc'] = tools.process_app_version(
            app = kwargs['app'],
            version_num = int(kwargs['version_num']),
            exec_files = kwargs['exec_files'],
            signature_files = kwargs.setdefault('signature_files',{}))
        del kwargs['signature_files']
        del kwargs['exec_files']
        del kwargs['exec_file']
        database.AppVersion.__init__(self,**kwargs)

# format: [ database.Object, args, ...]
#   arg format:
#       'arg'
#       '?arg'    optional
#       [ 'arg', default_value ]

# NOTE TO KARL: If cross_project_id is not supplied, the default value
# should be a random 32 character script obtained by doing the md5sum
# hash of (say) 512 bytes from /dev/urandom or similar.

list_objects_to_add = [
    [ database.Platform,   'name', 'user_friendly_name', CREATE_TIME ],
    [ database.App,        'name', 'user_friendly_name', ['?min_version',0], CREATE_TIME],
    [ XAppVersion, 'app', 'platform', 'version_num', 'exec_file', '?signature_file',
      CREATE_TIME ],
    [ database.User,       'name', 'email_addr', 'authenticator',
      ['?country','United States'], ['?postal_code','0'], ['?cross_project_id', '0'],
      '?global_prefs', '?global_prefs_file',
      CREATE_TIME ],
    # [ database.Workunit,   'zzzz' ],
    ]

class AddObject:
    pass

add_objects = {}
for o in list_objects_to_add:
    add_object = AddObject()
    add_object.DatabaseObject = o[0]
    add_object.name = add_object.DatabaseObject._table.table
    add_object.args = []
    add_object.optional_args = []
    add_object.default_values = {}
    for arg in o[1:]:
        if isinstance(arg, list):
            default_value = arg[1]
            arg = arg[0]
        else:
            default_value = None
        if arg.startswith('?'):
            optional = True
            arg = arg[1:]
        else:
            optional = False
        if optional:
            add_object.optional_args.append(arg)
        else:
            add_object.args.append(arg)
        if default_value:
            add_object.default_values[arg] = default_value
    add_objects[add_object.name] = add_object

most_recent_exec_file = None

def translate_arg(object, arg, value, args_dict):
    '''Translate various user argument values, for adding given ``object``.
    Modifies ``args_dict``.'''

    database_table = None
    try:
        database_table = database.__dict__[arg.capitalize()]._table
    except:
        pass
    if database_table:
        args_dict[arg] = translate_database_arg(database_table, arg, value)
        return

    if arg == 'global_prefs_file':
        args_dict['global_prefs'] = open(value).read()
        return

    if object.DatabaseObject == XAppVersion:
        # 'add app_version' accepts multiple '-exec_file's with
        # '-signature_file' applying to the most recent exec_file
        if arg == 'exec_file':
            global most_recent_exec_file
            most_recent_exec_file = value
            args_dict.setdefault('exec_files',[]).append(value)
            # since 'exec_file' (without 's') is required, set it to None so
            # that argument checker knows we got one; we'll delete it later.
            args_dict[arg] = None
            return
        if arg == 'signature_file':
            args_dict.setdefault('signature_files',{})[most_recent_exec_file] = value
            return

    if arg == 'cross_project_id':
        if not value:
            value = tools.make_uuid()

    args_dict[arg] = value

def translate_database_arg(database_table, arg, value):
    '''Look up an object ``value`` either as a database ID or string.
    This allows us to accept e.g. either --app Astropulse or --app 1'''
    try:
        id = int(value)
        results = database_table.find(id=id)
        if not results:
            raise Exception("")
    except:
        results = database_table.find(name=value)
    if len(results) == 0:
        raise SystemExit('No %s "%s" found' %(arg,value))
    if len(results) > 1:
        print('Too many %ss match "%s": '%(arg,value), sys.stderr)
        for result in results:
            print ('   '+result.name, sys.stderr)
        raise SystemExit
    return results[0]

class AddObjectException(Exception): pass

def check_required_arguments(add_object, args_dict):
    for arg in add_object.args:
        if not arg in args_dict:
            raise AddObjectException('required value for %s not given'%arg)

def translate_args_dict(add_object, untranslated_args_dict):
    args_dict = add_object.default_values.copy()
    for arg,value in untranslated_args_dict.items():
        translate_arg(add_object,arg,value,args_dict)
    return args_dict

def exception_is_duplicate_entry(exception):
    '''Checks a MySQLdb.IntegrityError for whether the error is Duplicate
    Entry.  Kludgy.'''
    return (isinstance(exception, MySQLdb.IntegrityError) and
            str(exception).find('Duplicate entry')!=-1)

def do_add_object(add_object, untranslated_args_dict, skip_old=False):
    '''Input ```args_dict``` must have been translated already.'''
    args_dict = translate_args_dict(add_object, untranslated_args_dict)
    check_required_arguments(add_object, args_dict)
    dbobject = add_object.DatabaseObject(**args_dict)
    print("Processing"+dbobject+"...")
    # print "Commiting", dbobject, "with args:"
    # pprint.pprint(dbobject.__dict__)
    try:
        dbobject.commit()
    except MySQLdb.MySQLError as e:
        if skip_old and exception_is_duplicate_entry(e):
            print("  Skipped existing"+dbobject)
            return
        else:
            raise SystemExit('Error committing %s: %s' %(dbobject, e))

    # delete object and re-select it from database to allow user to check
    # consistency
    id = dbobject.id
    del dbobject
    dbobject = add_object.DatabaseObject._table[id]
    print("  Committed"+dbobject+"; values:")
    pprint.pprint(dbobject.__dict__)



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import os
import requests
import sys

def get_all_yml_files():
    yml_files = []
    for root, _, files in os.walk(".github/workflows", topdown=True):
        for filename in files:
            if filename.endswith(".yml"):
                yml_files.append(os.path.join(root, filename))
    return yml_files

def get_all_actions(yml_files):
    actions = []
    for yml_file in yml_files:
        with open(yml_file, "r") as f:
            lines = f.readlines()
        for line in lines:
            if "uses: " in line:
                line = line.strip()
                line = line.split("@")[0]
                line = line.split("uses: ")[1]
                res = line.split(" ")
                if len(res) > 1 and res[1] == "# ignore-check":
                    continue
                line = res[0]
                if line not in actions:
                    actions.append(line)
    return actions

def get_action_with_sha(action, token):
    if token is not None:
        headers = {'Authorization': f'Bearer {token}'}
    else:
        headers = None
    print("Getting latest release for: " + action)
    res = requests.get(f'https://api.github.com/repos/{action}/releases/latest', headers=headers).json()
    tag_name = res['tag_name']
    res = requests.get(f'https://api.github.com/repos/{action}/git/ref/tags/{tag_name}', headers=headers).json()
    object_type = res['object']['type']
    tag_sha = res['object']['sha']
    if object_type != 'commit':
        res = requests.get(f'https://api.github.com/repos/{action}/git/tags/{tag_sha}', headers=headers).json()
        tag_sha = res['object']['sha']
    return action, tag_sha

def get_all_actions_with_sha(actions, token):
    actions_with_sha = []
    for action in actions:
        actions_with_sha.append(get_action_with_sha(action, token))
    return actions_with_sha

def get_action_sha(actions_with_sha, action):
    for action_with_sha in actions_with_sha:
        if action_with_sha[0] == action:
            return action_with_sha[1]

def process_yml_file(yml_file, actions_with_sha):
    print("Processing file: " + yml_file)
    with open(yml_file, "r") as f:
        lines = f.readlines()
    with open(yml_file, "w") as f:
        for line in lines:
            new_line = line
            if "uses: " in new_line:
                new_line = new_line.strip()
                res = new_line.split("@")
                version = res[1]
                new_line = res[0]
                new_line = new_line.split("uses: ")[1]
                res = version.split(" ")
                if len(res) == 1:
                    version = res[0]
                    new_sha = get_action_sha(actions_with_sha, new_line)
                    if new_sha != version:
                        line = line.replace(version, new_sha)
            f.write(line)

def process_all_yml_files(yml_files, actions_with_sha):
    for yml_file in yml_files:
        process_yml_file(yml_file, actions_with_sha)


if (len(sys.argv) > 1):
    token = sys.argv[1]
else:
    token = None

yml_files = get_all_yml_files()

latest_actions = get_all_actions_with_sha(get_all_actions(yml_files), token)

process_all_yml_files(yml_files, latest_actions)



# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2026 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

"""
Windows Installer Integration Tests Preparation Script

This script prepares the Windows environment for installer integration tests.
Currently supports creating test users on Windows.
"""

import argparse
import platform
import subprocess
import sys


def check_windows():
    """Verify that the script is running on Windows."""
    if platform.system() != 'Windows':
        print("Error: This script must be run on Windows.", file=sys.stderr)
        sys.exit(1)


def create_windows_user(username, password):
    """
    Create a new Windows user account.

    Args:
        username: The username for the new account
        password: The password for the new account

    Returns:
        bool: True if user was created successfully, False otherwise
    """
    try:
        # Use net user command to create a new local user account
        cmd = ['net', 'user', username, password, '/add', '/y']
        result = subprocess.run(
            cmd,
            capture_output=True,
            text=True,
            check=False
        )

        if result.returncode == 0:
            print(f"Successfully created user: {username}")
            return True
        else:
            error_msg = result.stderr.strip() or result.stdout.strip() or "Unknown error occurred"
            print(f"Error creating user: {error_msg}", file=sys.stderr)
            return False

    except Exception as e:
        print(f"Exception while creating user: {e}", file=sys.stderr)
        return False


def main():
    """Main entry point for the script."""
    parser = argparse.ArgumentParser(
        description='Prepare Windows environment for installer integration tests'
    )

    parser.add_argument(
        '--create-user',
        action='store_true',
        help='Create a new Windows user account'
    )

    parser.add_argument(
        '--username',
        type=str,
        help='Username for the new account (required with --create-user)'
    )

    parser.add_argument(
        '--password',
        type=str,
        help='Password for the new account (required with --create-user)'
    )

    args = parser.parse_args()

    # Check if running on Windows
    check_windows()

    # Validate arguments
    if args.create_user:
        if not args.username or not args.password:
            parser.error('--create-user requires both --username and --password')

        # Create the user
        success = create_windows_user(args.username, args.password)
        sys.exit(0 if success else 1)
    else:
        parser.print_help()
        sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2026 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib
import sys
import testset
import testhelper

class IntegrationTests:
    def __init__(self):
        self.testhelper = testhelper.TestHelper()
        self.result = True
        self.result &= self.test_files_exist()
        self.result &= self.test_version()
        self.result &= self.test_user()
        self.result &= self.test_selected_values_from_boinc_client_service_file()
        self.result &= self.test_files_permissions()

    def _resolve_path(self, path):
        path = os.path.abspath(path)
        components = path.split(os.sep)
        current_path = os.sep if path.startswith(os.sep) else components[0]
        symlinks_found = False
        for part in components[1:] if path.startswith(os.sep) else components[1:]:
            current_path = os.path.join(current_path, part)
            if os.path.islink(current_path):
                symlinks_found = True
        if symlinks_found:
            return os.path.realpath(path)
        else:
            return path

    def _get_test_executable_file_path(self, filename):
        path = os.popen("echo $PATH").read().strip()
        for p in path.split(":"):
            if os.path.exists(os.path.join(p, filename)):
                return self._resolve_path(os.path.join(p, filename))
        return ""

    def _get_file_version(self, filename):
        return self.testhelper.get_version_from_string(os.popen(("{app} --version").format(app=self._get_test_executable_file_path(filename))).read().strip())

    def _get_user_exists(self, username):
        return os.popen("id -un {username}".format(username=username)).read().strip() == username

    def _get_group_exists(self, groupname):
        return os.popen("getent group {groupname}".format(groupname=groupname)).read().strip().split(":")[0] == groupname

    def _get_user_in_group(self, username, groupname):
        return os.popen("id -Gn {username}".format(username=username)).read().strip().find(groupname) != -1

    def _get_user_home_directory(self, username):
        return os.popen("getent passwd {username}".format(username=username)).read().strip().split(":")[5]

    def _get_uid_range(self, username):
        result = os.popen("cat /etc/subuid | grep {username}".format(username=username)).read().strip()
        if (result == ""):
            return ""
        result = result.split(":")
        if (len(result) != 3):
            return ""
        start = int(result[1])
        count = int(result[2])
        return "{start}:{count}".format(start=start, count=count)

    def _get_gid_range(self, groupname):
        result = os.popen("cat /etc/subgid | grep {groupname}".format(groupname=groupname)).read().strip()
        if (result == ""):
            return ""
        result = result.split(":")
        if (len(result) != 3):
            return ""
        start = int(result[1])
        count = int(result[2])
        return "{start}:{count}".format(start=start, count=count)

    def _get_key_value_pairs_from_file(self, filename):
        result = {}
        with open(filename, "r") as f:
            lines = f.readlines()
            for line in lines:
                if (line.find("=") != -1):
                    key, value = line.split("=")
                    result[key.strip()] = value.strip()
        return result

    def _get_file_owner(self, filename):
        if os.path.islink(filename):
            path = pathlib.Path(os.readlink(filename))
        else:
            path = pathlib.Path(filename)
        return "{owner}:{group}".format(owner=path.owner(), group=path.group())

    def _get_ca_certificates_file_paths(self):
        paths = []
        if os.path.exists("/etc/ssl/certs/ca-certificates.crt"):
            paths.append("/etc/ssl/certs/ca-certificates.crt")
        if os.path.exists("/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem"):
            paths.append("/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem")
        if os.path.exists("/etc/ssl/ca-bundle.pem"):
            paths.append("/etc/ssl/ca-bundle.pem")
        return paths

    def test_files_exist(self):
        ts = testset.TestSet("Test files exist")
        ts.expect_equal("/usr/local/bin/boinc", self._get_test_executable_file_path("boinc"), "Test 'boinc' file location")
        ts.expect_equal("/usr/local/bin/boinccmd", self._get_test_executable_file_path("boinccmd"), "Test 'boinccmd' file location")
        ts.expect_equal("/usr/local/bin/boincmgr", self._get_test_executable_file_path("boincmgr"), "Test 'boincmgr' file location")
        ts.expect_true(os.path.exists("/usr/lib/systemd/system/boinc-client.service"), "Test 'boinc-client.service' file exists in '/usr/lib/systemd/system/'")
        ts.expect_true(os.path.exists("/etc/default/boinc-client"), "Test 'boinc-client' file exists in '/etc/default/'")
        ts.expect_true(os.path.exists("/etc/init.d/boinc-client"), "Test 'boinc-client' file exists in '/etc/init.d/'")
        ts.expect_true(os.path.exists("/etc/bash_completion.d/boinc.bash"), "Test 'boinc.bash' file exists in '/etc/bash_completion.d/'")
        ts.expect_true(os.path.exists("/etc/X11/Xsession.d/36x11-common_xhost-boinc"), "Test '36x11-common_xhost-boinc' file exists in '/etc/X11/Xsession.d/'")
        ts.expect_true(os.path.exists("/usr/local/share/applications/boinc.desktop"), "Test 'boinc.desktop' file exists in '/usr/local/share/applications/'")
        ts.expect_true(os.path.exists("/usr/local/share/icons/boinc.png"), "Test 'boinc.png' file exists in '/usr/local/share/icons/'")
        ts.expect_true(os.path.exists("/usr/local/share/icons/boinc.svg"), "Test 'boinc.svg' file exists in '/usr/local/share/icons/'")
        ts.expect_true(os.path.exists("/var/lib/boinc/cc_config.xml"), "Test 'cc_config.xml' file exists in '/var/lib/boinc/'")
        ts.expect_true(os.path.exists("/etc/boinc-client/cc_config.xml"), "Test 'cc_config.xml' file exists in '/etc/boinc-client/'")
        if (os.path.islink("/etc/boinc-client/cc_config.xml")):
            ts.expect_equal("/var/lib/boinc/cc_config.xml", os.readlink("/etc/boinc-client/cc_config.xml"), "Test '/etc/boinc-client/cc_config.xml' file is a symbolic link to '/var/lib/boinc/cc_config.xml'")
        elif(os.path.islink("/var/lib/boinc/cc_config.xml")):
            ts.expect_equal("/etc/boinc-client/cc_config.xml", os.readlink("/var/lib/boinc/cc_config.xml"), "Test '/var/lib/boinc/cc_config.xml' file is a symbolic link to '/etc/boinc-client/cc_config.xml'")
        else:
            ts.expect_true(False, "Test 'cc_config.xml' file is a symbolic link")
        ts.expect_true(os.path.exists("/var/lib/boinc/global_prefs_override.xml"), "Test 'global_prefs_override.xml' file exists in '/var/lib/boinc/'")
        ts.expect_true(os.path.exists("/etc/boinc-client/global_prefs_override.xml"), "Test 'global_prefs_override.xml' file exists in '/etc/boinc-client/'")
        if (os.path.islink("/etc/boinc-client/global_prefs_override.xml")):
            ts.expect_equal("/var/lib/boinc/global_prefs_override.xml", os.readlink("/etc/boinc-client/global_prefs_override.xml"), "Test '/etc/boinc-client/global_prefs_override.xml' file is a symbolic link to '/var/lib/boinc/global_prefs_override.xml'")
        elif(os.path.islink("/var/lib/boinc/global_prefs_override.xml")):
            ts.expect_equal("/etc/boinc-client/global_prefs_override.xml", os.readlink("/var/lib/boinc/global_prefs_override.xml"), "Test '/var/lib/boinc/global_prefs_override.xml' file is a symbolic link to '/etc/boinc-client/global_prefs_override.xml'")
        else:
            ts.expect_true(False, "Test 'global_prefs_override.xml' file is a symbolic link")
        ts.expect_true(os.path.exists("/var/lib/boinc/remote_hosts.cfg"), "Test 'remote_hosts.cfg' file exists in '/var/lib/boinc/'")
        ts.expect_true(os.path.exists("/etc/boinc-client/remote_hosts.cfg"), "Test 'remote_hosts.cfg' file exists in '/etc/boinc-client/'")
        if (os.path.islink("/etc/boinc-client/remote_hosts.cfg")):
            ts.expect_equal("/var/lib/boinc/remote_hosts.cfg", os.readlink("/etc/boinc-client/remote_hosts.cfg"), "Test '/etc/boinc-client/remote_hosts.cfg' file is a symbolic link to '/var/lib/boinc/remote_hosts.cfg'")
        elif(os.path.islink("/var/lib/boinc/remote_hosts.cfg")):
            ts.expect_equal("/etc/boinc-client/remote_hosts.cfg", os.readlink("/var/lib/boinc/remote_hosts.cfg"), "Test '/var/lib/boinc/remote_hosts.cfg' file is a symbolic link to '/etc/boinc-client/remote_hosts.cfg'")
        else:
            ts.expect_true(False, "Test 'remote_hosts.cfg' file is a symbolic link")
        ts.expect_true(os.path.exists("/var/lib/boinc/gui_rpc_auth.cfg"), "Test 'gui_rpc_auth.cfg' file exists in '/var/lib/boinc/'")
        ts.expect_true(os.path.exists("/etc/boinc-client/gui_rpc_auth.cfg"), "Test 'gui_rpc_auth.cfg' file exists in '/etc/boinc-client/'")
        if (os.path.islink("/etc/boinc-client/gui_rpc_auth.cfg")):
            ts.expect_equal("/var/lib/boinc/gui_rpc_auth.cfg", os.readlink("/etc/boinc-client/gui_rpc_auth.cfg"), "Test '/etc/boinc-client/gui_rpc_auth.cfg' file is a symbolic link to '/var/lib/boinc/gui_rpc_auth.cfg'")
        elif(os.path.islink("/var/lib/boinc/gui_rpc_auth.cfg")):
            ts.expect_equal("/etc/boinc-client/gui_rpc_auth.cfg", os.readlink("/var/lib/boinc/gui_rpc_auth.cfg"), "Test '/var/lib/boinc/gui_rpc_auth.cfg' file is a symbolic link to '/etc/boinc-client/gui_rpc_auth.cfg'")
        else:
            ts.expect_true(False, "Test 'gui_rpc_auth.cfg' file is a symbolic link")
        ts.expect_true(len(self._get_ca_certificates_file_paths()) > 0, "Test system 'ca-certificates.crt' file exists")
        ts.expect_true(os.path.exists("/var/lib/boinc/ca-bundle.crt"), "Test 'ca-bundle.crt' file exists in '/var/lib/boinc/'")
        ts.expect_true(os.path.islink("/var/lib/boinc/ca-bundle.crt"), "Test '/var/lib/boinc/ca-bundle.crt' file is a symbolic link")
        ts.expect_true(os.readlink("/var/lib/boinc/ca-bundle.crt") in self._get_ca_certificates_file_paths(), "Test '/var/lib/boinc/ca-bundle.crt' file is a symbolic link to the system 'ca-certificates.crt' file")
        ts.expect_true(os.path.exists("/var/lib/boinc/all_projects_list.xml"), "Test 'all_projects_list.xml' file exists in '/var/lib/boinc/'")
        return ts.result()

    def test_version(self):
        ts = testset.TestSet("Test version is correct")
        current_version = self.testhelper.get_current_version_number()
        ts.expect_not_equal("", current_version, "Test current version could be read from the 'version.h' file")
        ts.expect_equal(current_version, self._get_file_version("boinc"), "Test 'boinc' version is correctly set")
        ts.expect_equal(current_version, self._get_file_version("boinccmd"), "Test 'boinccmd' version is correctly set")
        return ts.result()

    def test_user(self):
        ts = testset.TestSet("Test 'boinc' user and 'boinc' group exist")
        ts.expect_true(self._get_user_exists("boinc"), "Test 'boinc' user exists")
        ts.expect_true(self._get_group_exists("boinc"), "Test 'boinc' group exists")
        ts.expect_true(self._get_user_in_group("boinc", "boinc"), "Test 'boinc' user is in 'boinc' group")
        if (self._get_group_exists("video")):
            ts.expect_true(self._get_user_in_group("boinc", "video"), "Test 'boinc' user is in 'video' group")
        if (self._get_group_exists("render")):
            ts.expect_true(self._get_user_in_group("boinc", "render"), "Test 'boinc' user is in 'render' group")
        if (self._get_group_exists("docker")):
            ts.expect_true(self._get_user_in_group("boinc", "docker"), "Test 'boinc' user is in 'docker' group")
        ts.expect_equal("/var/lib/boinc", self._get_user_home_directory("boinc"), "Test 'boinc' user home directory is '/var/lib/boinc'")
        ts.expect_equal("100000:65536", self._get_uid_range("boinc"), "Test 'boinc' user UID range is '100000:65536'")
        ts.expect_equal("100000:65536", self._get_gid_range("boinc"), "Test 'boinc' group GID range is '100000:65536'")
        return ts.result()

    def test_selected_values_from_boinc_client_service_file(self):
        ts = testset.TestSet("Test selected values from the '/usr/lib/systemd/system/boinc-client.service' file")
        data = self._get_key_value_pairs_from_file("/usr/lib/systemd/system/boinc-client.service")
        ts.expect_equal(data["ProtectSystem"], "strict", "Test 'ProtectSystem' is correctly set")
        ts.expect_equal(data["ReadWritePaths"], "-/var/lib/boinc -/etc/boinc-client -/tmp -/var/tmp", "Test 'ReadWritePaths' is correctly set")
        ts.expect_equal(data["User"], "boinc", "Test 'User' is correctly set")
        ts.expect_equal(data["WorkingDirectory"], "/var/lib/boinc", "Test 'WorkingDirectory' is correctly set")
        ts.expect_equal(data["ExecStart"], "/usr/local/bin/boinc", "Test 'ExecStart' is correctly set")
        ts.expect_equal(data["ExecStop"], "/usr/local/bin/boinccmd --quit", "Test 'ExecStop' is correctly set")
        ts.expect_equal(data["ExecReload"], "/usr/local/bin/boinccmd --read_cc_config", "Test 'ExecReload' is correctly set")
        ts.expect_equal(data["ExecStopPost"], "/bin/rm -f lockfile", "Test 'ExecStopPost' is correctly set")
        return ts.result()

    def test_files_permissions(self):
        ts = testset.TestSet("Test files permissions")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/etc/boinc-client/cc_config.xml"), "Test '/etc/boinc-client/cc_config.xml' file owner")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/etc/boinc-client/global_prefs_override.xml"), "Test '/etc/boinc-client/global_prefs_override.xml' file owner")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/etc/boinc-client/remote_hosts.cfg"), "Test '/etc/boinc-client/remote_hosts.cfg' file owner")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/var/lib/boinc/cc_config.xml"), "Test '/var/lib/boinc/cc_config.xml' file owner")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/var/lib/boinc/global_prefs_override.xml"), "Test '/var/lib/boinc/global_prefs_override.xml' file owner")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/var/lib/boinc/remote_hosts.cfg"), "Test '/var/lib/boinc/remote_hosts.cfg' file owner")
        ts.expect_equal("boinc:boinc", self._get_file_owner("/var/lib/boinc/gui_rpc_auth.cfg"), "Test '/var/lib/boinc/gui_rpc_auth.cfg' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/local/bin/boinc"), "Test '/usr/local/bin/boinc' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/local/bin/boinccmd"), "Test '/usr/local/bin/boinccmd' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/local/bin/boincmgr"), "Test '/usr/local/bin/boincmgr' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/lib/systemd/system/boinc-client.service"), "Test '/usr/lib/systemd/system/boinc-client.service' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/etc/default/boinc-client"), "Test '/etc/default/boinc-client' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/etc/init.d/boinc-client"), "Test '/etc/init.d/boinc-client' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/etc/bash_completion.d/boinc.bash"), "Test '/etc/bash_completion.d/boinc.bash' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/etc/X11/Xsession.d/36x11-common_xhost-boinc"), "Test '/etc/X11/Xsession.d/36x11-common_xhost-boinc' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/local/share/applications/boinc.desktop"), "Test '/usr/local/share/applications/boinc.desktop' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/local/share/icons/boinc.png"), "Test '/usr/local/share/icons/boinc.png' file owner")
        ts.expect_equal("root:root", self._get_file_owner("/usr/local/share/icons/boinc.svg"), "Test '/usr/local/share/icons/boinc.svg' file owner")
        return ts.result()

if __name__ == "__main__":
    if not IntegrationTests().result:
        sys.exit(1)
    sys.exit(0)



## $Id$

'''
Defines database backend library and database table and object relationships.

Example usage:

import database, db_mid

# get platform with id 7; will raise exception if no such platform.
p7 = database.Platforms[7]

# get platforms with friendly name "commodore 64"
p_c64 = database.Platforms.find(user_friendly_name="commodore 64")

# delete results of workunit with name "dead.wu", and email their users:
wu_dead = database.Workunits.find(name="dead.wu")[0]
results_dead = database.Results.find(wu=wu_dead)
for result in results_dead:
    print("Removing from db:", result)
    os.system("echo oeps | mail %s" % result.host.user.email_addr)
    result.remove()

# multiply the total_credit of each user by 17:
for user in database.Users.find():
    user.total_credit *= 17
    user.commit()

'''

from __future__ import print_function
from Boinc import configxml
from Boinc.util import *
from Boinc.db_base import *

ID = '$Id$'

class Platform(DatabaseObject):
    _table = DatabaseTable(
        table = 'platform',
        columns = [ 'create_time',
                    'name',
                    'user_friendly_name',
                    'deprecated' ])

class App(DatabaseObject):
    _table = DatabaseTable(
        table = 'app',
        columns = [ 'create_time',
                    'name',
                    'min_version',
                    'deprecated',
                    'user_friendly_name',
                    'homogeneous_redundancy',
                    'weight',
                    'beta',
                    'target_nresults',
                    'min_avg_pfc',
                    'host_scale_check',
                    'homogeneous_app_version',
                    'non_cpu_intensive'
                    ])

class AppVersion(DatabaseObject):
    _table = DatabaseTable(
        table = 'app_version',
        columns = [ 'create_time',
                    'appid',
                    'version_num',
                    'platformid',
                    'xml_doc',
                    'min_core_version',
                    'max_core_version',
                    'deprecated',
                    'plan_class',
                    'pfc_n',
                    'pfc_avg',
                    'pfc_scale',
                    'expavg_credit',
                    'expavg_time',
                    'beta'
                    ])

class User(DatabaseObject):
    _table = DatabaseTable(
        table = 'user',
        columns = [ 'create_time',
                    'email_addr',
                    'name',
                    'authenticator',
                    'country',
                    'postal_code',
                    'total_credit',
                    'expavg_credit',
                    'expavg_time',
                    'global_prefs',
                    'project_prefs',
                    'teamid',
                    'venue',
                    'url',
                    'send_email',
                    'show_hosts',
                    'posts',
                    'seti_id',
                    'seti_nresults',
                    'seti_last_result_time',
                    'seti_total_cpu',
                    'signature',
                    'has_profile',
                    'cross_project_id',
                    'passwd_hash',
                    'email_validated',
                    'donated'
                    ])

class Team(DatabaseObject):
    _table = DatabaseTable(
        table = 'team',
        columns = [ 'create_time',
                    'userid',
                    'name',
                    'name_lc',
                    'url',
                    'type',
                    'name_html',
                    'description',
                    'nusers',
                    'country',
                    'total_credit',
                    'expavg_credit',
                    'expavg_time',
                    'seti_id',
                    'ping_user',
                    'ping_time'
                    ])

class Host(DatabaseObject):
    _table = DatabaseTable(
        table = 'host',
        columns = [ 'create_time',
                    'userid',
                    'rpc_seqno',
                    'rpc_time',
                    'total_credit',
                    'expavg_credit',
                    'expavg_time',
                    'timezone',
                    'domain_name',
                    'serialnum',
                    'last_ip_addr',
                    'nsame_ip_addr',
                    'on_frac',
                    'connected_frac',
                    'active_frac',
                    'p_ncpus',
                    'p_vendor',
                    'p_model',
                    'p_fpops',
                    'p_iops',
                    'p_membw',
                    'os_name',
                    'os_version',
                    'm_nbytes',
                    'm_cache',
                    'm_swap',
                    'd_total',
                    'd_free',
                    'd_boinc_used_total',
                    'd_boinc_used_project',
                    'd_boinc_max',
                    'n_bwup',
                    'n_bwdown',
                    'credit_per_cpu_sec',
                    'venue',
                    'projects',
                    'nresults_today',
                    'avg_turnaround',
                    'host_cpid',
                    'external_ip_addr',
                    'max_results_day'
                    ])

class Workunit(DatabaseObject):
    _table = DatabaseTable(
        table = 'workunit',
        columns = [ 'create_time',
                    'appid',
                    'name',
                    'xml_doc',
                    'batch',
                    'rsc_fpops_est',
                    'rsc_fpops_bound',
                    'rsc_memory_bound',
                    'rsc_disk_bound',
                    'need_validate',
                    'canonical_resultid',
                    'canonical_credit',
                    'transition_time',
                    'delay_bound',
                    'error_mask',
                    'file_delete_state',
                    'assimilate_state',
                    'hr_class',
                    'opaque',
                    'min_quorum',
                    'target_nresults',
                    'max_error_results',
                    'max_total_results',
                    'max_success_results',
                    'result_template_file',
                    'priority',
                    'mod_time'
                    ])

class Result(DatabaseObject):
    _table = DatabaseTable(
        table = 'result',
        columns = [ 'create_time',
                    'workunitid',
                    'server_state',
                    'outcome',
                    'client_state',
                    'hostid',
                    'userid',
                    'report_deadline',
                    'sent_time',
                    'received_time',
                    'name',
                    'cpu_time',
                    'xml_doc_in',
                    'xml_doc_out',
                    'stderr_out',
                    'batch',
                    'file_delete_state',
                    'validate_state',
                    'claimed_credit',
                    'granted_credit',
                    'opaque',
                    'random',
                    'client_version_num',
                    'appid',
                    'teamid',
                    'priority',
                    'mod_time'
                    ])


def connect(config = None, nodb = False):
    """Connect if not already connected, using config values."""
    if get_dbconnection():
        return 0
    config = config or configxml.default_config().config
    if nodb:
        db = ''
    else:
        db = config.db_name

    host=config.__dict__.get('db_host','')
    port=""
    if ':' in host:
        host,port=config.__dict__.get('db_host','').split(":")

    if port == '':
        port = 3306
    else:
        port = int(port)
    do_connect(db=db,
               host=host,
               port=port,
               user=config.__dict__.get('db_user',''),
               passwd=config.__dict__.get('db_passwd', ''))
    return 1

def _execute_sql_script(cursor, filename):
    for query in open(filename).read().split(';'):
        query = query.strip()
        if not query: continue
        cursor.execute(query)

def create_database(srcdir, config = None, drop_first = False):
    ''' creates a new database. '''
    import boinc_path_config
    config = config or configxml.default_config().config
    connect(config, nodb=True)
    dbcon = get_dbconnection()
    cursor = dbcon.cursor()
    if drop_first:
        cursor.execute("drop database if exists %s"%config.db_name)
    cursor.execute("create database %s character set utf8mb4 collate utf8mb4_unicode_ci"%config.db_name)
    cursor.execute("use %s"%config.db_name)
    for file in ['schema.sql', 'constraints.sql', 'content.sql']:
        _execute_sql_script(cursor, os.path.join(srcdir, 'db', file))
    dbcon.commit()
    cursor.close()

# alias
connect_default_config = connect

database_classes_ = [ Platform,
                      App,
                      AppVersion,
                      User,
                      Team,
                      Host,
                      Workunit,
                      Result ]

Platforms    = Platform._table
Apps         = App._table
AppVersions  = AppVersion._table
Users        = User._table
Teams        = Team._table
Hosts        = Host._table
Workunits    = Workunit._table
Results      = Result._table

init_table_classes(database_classes_,{'canonical_result': Result})



# module for setting up a new project (either a real project or a test project
# see tools/makeproject, test/testbase.py).

# TODO: make sure things work if build_dir != src_dir

from __future__ import print_function
import boinc_path_config
from Boinc import database, db_mid, configxml, tools
from Boinc.boinc_db import *
import os, sys, glob, time, shutil, re, random

class Options:
    pass

options = Options()
errors = Options()
errors.count = 0

options.have_init = False
options.install_method = None
options.is_test = False
options.drop_db_first = False

def init():
    if options.have_init: return
    options.have_init = True

    if options.install_method == 'copy':
        options.install_function = shutil.copy
    elif options.install_method == 'link' or options.install_method == 'hardlink':
        options.install_function = my_link
    elif options.install_method == 'symlink' or options.install_method == 'softlink':
        options.install_function = my_symlink
    else:
        fatal_error("Invalid install method: %s"%options.install_method)

def verbose_echo(level, line):
    print(line)
    sys.stdout.flush()

def fatal_error(msg):
    errors.count += 1
    verbose_echo(0, "FATAL ERROR: "+msg)
    sys.exit(1)

def error(msg, fatal=0):
    if fatal: fatal_error(msg)
    errors.count += 1
    verbose_echo(0, "ERROR: "+msg)

def verbose_sleep(msg, wait):
    front = msg + ' [sleep '
    back = ']'
    for i in range(1,wait+1):
        verbose_echo(1, msg + ' [sleep ' + ('.'*i).ljust(wait) + ']')
        time.sleep(1)

def get_env_var(name, default = None):
    value = os.environ.get(name, default)
    if value is None:
        print("Environment variable %s not defined" % name)
        sys.exit(1)
    return value

def shell_call(cmd, doexec=False, failok=False):
    if doexec:
        os.execl('/bin/sh', 'sh', '-c', cmd)
        error("Command failed: "+cmd, fatal=(not failok))
        os._exit(1)
    if os.system(cmd):
        error("Command failed: "+cmd, fatal=(not failok))
        return 1
    return 0

def verbose_shell_call(cmd, doexec=False, failok=False):
    verbose_echo(2, "   "+cmd)
    return shell_call(cmd, doexec, failok)

def destpath(src,dest):
    if dest.endswith('/'):
        return dest + os.path.basename(src)
    else:
        return dest

# my_symlink and my_link just add the filename to the exception object if one
# is raised - don't know why it's not already there
def my_symlink(src,dest):
    dest = destpath(src,dest)
    try:
        os.symlink(src,dest)
    except OSError as e:
        e.filename = src + ' -> ' + dest
        raise

def my_link(src,dest):
    dest = destpath(src,dest)
    try:
        os.link(src,dest)
    except OSError as e:
        e.filename = src + ' -> ' + dest
        raise

# install = options.install_function
def install(src, dest, unless_exists=False):
    if unless_exists and os.path.exists(dest):
        return
    try:
        options.install_function(src, dest)
    except:
        print('failed to copy ' + src + ' to ' + dest)
        return


def install_glob(glob_source, dest, failok=False):
    dest = os.path.join(dest, '') # append '/' if necessary
    for src in glob.glob(glob_source):
        if not os.path.isdir(src):
            install(src, dest)

def macro_substitute(macro, replacement, infile, outfile):
    open(outfile, 'w').write(open(infile).read().replace(macro, replacement))
def macro_substitute_inplace(macro, replacement, inoutfile):
    old = inoutfile + '.old'
    os.rename(inoutfile, old)
    macro_substitute(macro, replacement, old, inoutfile)

def check_program_exists(prog):
    if not os.path.isfile(prog):
        fatal_error("""
Executable not found: %s
Did you `make' yet?
""" % prog)
def check_core_client_executable():
    check_program_exists(builddir('client', version.CLIENT_BIN_FILENAME))
def check_app_executable(app):
    check_program_exists(builddir('apps', app))

def make_executable(name):
    os.chmod(name, 755)
def force_symlink(src, dest):
    if os.path.exists(dest):
        os.unlink(dest)
    my_symlink(src, dest)
def rmtree(dir):
    # if os.path.exists(dir):
    #     shutil.rmtree(dir)
    if not dir or dir == '/' or dir == '.' or ' ' in dir:
        raise Exception
    os.system("rm -rf %s"%dir)

def _remove_trail(s, suffix):
    if s.endswith(suffix):
        return s[:-len(suffix)]
    else:
        return s

def _url_to_filename(url):
    s=""
    for c in url.replace('http://',''):
        if (c.isalnum()):
            s += c
        else:
            s += '_'
    return _remove_trail(s,'_')

def account_file_name(url):
    return 'account_' + _url_to_filename(url) + '.xml'

def srcdir(location):
    return os.path.join(options.srcdir, location)

def builddir(location):
    return os.path.join(boinc_path_config.TOP_BUILD_DIR, location)

def run_tool(cmd):
    verbose_shell_call(builddir('tools', cmd))

def _gen_key_p(private_key, public_key):
    shell_call("%s/crypt_prog -genkey 1024 %s %s >/dev/null" % (
        builddir('lib'),
        private_key,
        public_key))
def _gen_key(key):
    _gen_key_p(key+'_private', key+'_public')

def get_int(s):
    '''Convert a string to an int; return 0 on error.'''
    try: return int(s)
    except: return 0

def unique(list):
    d = {}
    for i in list:
        d[i] = 1
    return d.keys()

def map_xml(dic, keys):
    if not isinstance(dic,dict):
        dic = dic.__dict__
    s = ''
    for key in keys:
        s += "<%s>%s</%s>\n" % (key, dic[key], key)
    return s[:-1]

def generate_shmem_key():
    return '0x1111%x' % random.randrange(0,2**16)

def _check_vars(dict, **names):
    for key in names:
        value = names[key]
        if not key in dict:
            if value is None:
                raise SystemExit('error in test script: required parameter "%s" not specified'%key)
            dict[key] = value
    for key in dict:
        if not key in names:
            raise SystemExit('error in test script: extraneous parameter "%s" unknown'%key)

# CAN REMOVE THE FOLLOWING 8 FNS??

# def db_query(db, query):
#     db.query(query)
#     result = db.use_result()
#     return result and result.fetch_row(0,1)

def num_results():
    return database.Results.count()
def num_results_unsent():
    return database.Results.count(server_state = RESULT_SERVER_STATE_UNSENT)
def num_results_in_progress():
    return database.Results.count(server_state = RESULT_SERVER_STATE_IN_PROGRESS)
def num_results_over():
    return database.Results.count(server_state = RESULT_SERVER_STATE_OVER)
def num_wus():
    return database.Workunits.count()
def num_wus_assimilated():
    return database.Workunits.count(assimilate_state = ASSIMILATE_DONE)
def num_wus_to_transition():
    return database.Workunits.count(_extra_params = ['transition_time<%d'%(time.time()+30*86400)])

def build_command_line(cmd, kwargs):
    for (key, value) in kwargs:
        cmd += " -%s '%s'" %(key,value)
    return cmd

def create_project_dirs(dest_dir):
    def mkdir2(d):
        try:
            os.makedirs(d)
        except OSError as e:
            if not os.path.isdir(d):
                raise SystemExit(e)

    directories = ('',
                   'cgi-bin',
                   'bin',
                   'py',
                   'py/Boinc',
                   'templates',
                   'upload',
                   'download',
                   'apps',
                   'html',
                   'html/cache',
                   'html/inc',
                   'html/inc/password_compat',
                   'html/inc/random_compat',
                   'html/inc/ReCaptcha',
                   'html/inc/ReCaptcha/RequestMethod',
                   'html/languages',
                   'html/languages/compiled',
                   'html/languages/translations',
                   'html/languages/project_specific_translations',
                   'html/ops',
                   'html/ops/ffmail',
                   'html/ops/mass_email',
                   'html/ops/remind_email',
                   'html/project',
                   'html/stats',
                   'html/user',
                   'html/user/img',
                   'html/user_profile',
                   'html/user_profile/images'
    )
    [ mkdir2(os.path.join(dest_dir, x)) for x in directories ]

    # For all directories that apache will put files in,
    # make them group-writeable and setGID.
    # Assuming that the "apache" user belongs to our primary group,
    # any files or dirs created by apache will be owned by
    # our primary group (not Apache's).
    #
    directories =  [
        'upload',
        'html/cache',
        'html/inc',
        'html/languages',
        'html/languages/compiled',
        'html/user_profile/images',
    ]
    for d in directories:
        os.chmod(os.path.join(dest_dir, d), 0o2770)

def install_boinc_files(dest_dir, install_web_files, install_server_files):
    """Copy files from source dir to project dir.
        Used by the upgrade script, so don't copy sample files to real name."""

    def dest(*dirs):
        location = dest_dir
        for d in dirs:
            location = os.path.join(location, d )
        return location

    create_project_dirs(dest_dir)

    # copy html/ops files in all cases.
    # The critical one is db_update.php,
    # which is needed even for a server_only upgrade
    install_glob(srcdir('html/ops/*.php'), dest('html/ops/'))

    if install_web_files:
        install_glob(srcdir('html/inc/*.inc'), dest('html/inc/'))
        install_glob(srcdir('html/inc/*.php'), dest('html/inc/'))
        install_glob(srcdir('html/inc/password_compat/*.inc'), dest('html/inc/password_compat/'))
        install_glob(srcdir('html/inc/random_compat/*.inc'), dest('html/inc/random_compat/'))
        install_glob(srcdir('html/inc/ReCaptcha/*.php'), dest('html/inc/ReCaptcha/'))
        install_glob(srcdir('html/inc/ReCaptcha/RequestMethod/*.php'), dest('html/inc/ReCaptcha/RequestMethod'))
        install_glob(srcdir('html/inc/*.dat'), dest('html/inc/'))
        install_glob(srcdir('html/ops/*.css'), dest('html/ops/'))
        install_glob(srcdir('html/ops/ffmail/sample*'), dest('html/ops/ffmail/'))
        install_glob(srcdir('html/ops/mass_email/sample*'), dest('html/ops/mass_email/'))
        install_glob(srcdir('html/ops/remind_email/sample*'), dest('html/ops/remind_email/'))
        install_glob(srcdir('html/user/*.php'), dest('html/user/'))
        install_glob(srcdir('html/user/*.inc'), dest('html/user/'))
        install_glob(srcdir('html/user/*.css'), dest('html/user/'))
        install_glob(srcdir('html/user/*.txt'), dest('html/user/'))
        install_glob(srcdir('html/user/*.js'), dest('html/user/'))
        install_glob(srcdir('html/user/*.png'), dest('html/user/img'))
        install_glob(srcdir('html/user/*.gif'), dest('html/user/img'))
        install_glob(srcdir('html/user/img/*.*'), dest('html/user/img'))
        if not os.path.exists(dest('html/user/motd.php')):
            shutil.copy(srcdir('html/user/sample_motd.php'), dest('html/user/motd.php'))
        os.system("rm -f "+dest('html/languages/translations/*'))
        install_glob(srcdir('html/languages/translations/*.po'), dest('html/languages/translations/'))

    # copy Python stuff
    install(srcdir('sched/start' ), dest('bin/start' ))
    force_symlink(dest('bin', 'start'), dest('bin', 'stop'))
    force_symlink(dest('bin', 'start'), dest('bin', 'status'))
    python_files = [
        '__init__.py',
        'add_util.py',
        'boinc_db.py',
        'boinc_project_path.py',
        'boincxml.py',
        'configxml.py',
        'database.py',
        'db_base.py',
        'db_mid.py',
        'projectxml.py',
        'sched_messages.py',
        'tools.py',
        'util.py'
    ]
    for s in python_files:
        install(srcdir("py/Boinc/" + s), dest('py/Boinc', s))

    content = '''
# Generated by make_project
import sys, os
sys.path.insert(0, os.path.join('{dest_dir}', 'py'))
'''.format(dest_dir=dest_dir)
    f = open(dest('bin', 'boinc_path_config.py'), "w")
    f.write(content)
    f.close()

    if not install_server_files:
        return

    # copy backend (C++) programs;
    # rename current web daemons in case they're in use

    if os.path.isfile(dest('cgi-bin', 'cgi')):
        os.rename(dest('cgi-bin', 'cgi'), dest('cgi-bin', 'cgi.old'))
    if os.path.isfile(dest('cgi-bin', 'fcgi')):
        os.rename(dest('cgi-bin', 'fcgi'), dest('cgi-bin', 'fcgi.old'))
        install(builddir('sched','fcgi'), dest('cgi-bin','fcgi'))
    if os.path.isfile(dest('cgi-bin', 'file_upload_handler')):
        os.rename(dest('cgi-bin', 'file_upload_handler'), dest('cgi-bin', 'file_upload_handler.old'))
    cgi_script = [ 'cgi', 'file_upload_handler']
    for f in cgi_script:
        install(builddir('sched/' + f), dest('cgi-bin',f))
    command = [
            'adjust_user_priority',
            'antique_file_deleter',
            'assimilator.py',
            'census',
            'db_dump',
            'db_purge',
            'delete_file',
            'feeder',
            'file_deleter',
            'get_file',
            'make_work',
            'pshelper',
            'put_file',
            'pymw_assimilator.py',
            'batch_collect_assimilator',
            'sample_bitwise_validator',
            'sample_dummy_assimilator',
            'sample_substr_validator',
            'sample_trivial_validator',
            'sample_work_generator',
            'script_assimilator',
            'script_validator',
            'show_shmem',
            'single_job_assimilator',
            'size_regulator',
            'transitioner',
            'transitioner_catchup.php',
            'trickle_credit',
            'trickle_deadline',
            'trickle_echo',
            'update_stats',
            'wu_check',
        ]
    for f in command:
        install(builddir('sched/' + f), dest('bin',f))
    command = [ 'vda', 'vdad' ]
    for f in command:
        install(builddir('vda/' + f), dest('bin',f))
    command = [
            'appmgr',
            'boinc_submit',
            'cancel_jobs',
            'create_work',
            'dbcheck_files_exist',
            'dir_hier_move',
            'dir_hier_path',
            'grep_logs',
            'manage_privileges',
            'parse_config',
            'query_job',
            'run_in_ops',
            'batch_collect_assimilate.py',
            'sign_executable',
            'stage_file',
            'stage_file_native',
            'submit_batch',
            'submit_buda',
            'submit_job',
            'update_versions',
            'xadd',
        ]
    for f in command:
        install(builddir('tools/' + f), dest('bin',f))
    install(srcdir('lib/crypt_prog'), dest('bin','crypt_prog'))
    install(srcdir('sched/db_dump_spec.xml' ), dest('','db_dump_spec.xml' ))

class Project:
    def __init__(self,
                 short_name, long_name,
                 cgi_url,
                 project_dir=None, key_dir=None,
                 master_url=None,
                 db_name=None,
                 host=None,
                 web_only=False,
                 no_db=False,
                 production=False
                 ):
        init()

        self.production     = production
        self.web_only       = web_only
        self.no_db          = no_db
        self.short_name     = short_name
        self.long_name      = long_name or 'Project ' + self.short_name.replace('_',' ').capitalize()

        self.project_dir   = project_dir or os.path.join(options.projects_dir, self.short_name)

        self.config = configxml.ConfigFile(self.dest('config.xml')).init_empty()
        config = self.config.config

        # this is where default project config is defined

        config.long_name = self.long_name
        config.db_user = options.db_user
        config.db_name = db_name or options.user_name + '_' + self.short_name
        config.db_passwd = options.db_passwd
        config.db_host = options.db_host
        config.shmem_key = generate_shmem_key()
        config.uldl_dir_fanout = 1024
        config.host = host
        config.min_sendwork_interval = 0
        config.max_wus_to_send = 50
        config.daily_result_quota = 500
        config.disable_account_creation = 0
        config.disable_account_creation_rpc = 0
        config.account_creation_rpc_require_consent = 0
        config.disable_web_account_creation = 0
        config.enable_login_mustagree_termsofuse = 0
        config.enable_privacy_by_default = 0
        config.show_results = 1
        config.cache_md5_info = 1
        config.sched_debug_level = 3
        config.fuh_debug_level = 3
        config.one_result_per_user_per_wu = 0
        config.send_result_abort = 1
        config.dont_generate_upload_certificates = 1
        config.ignore_upload_certificates = 1
        config.enable_delete_account = 0
        if web_only:
            config.no_computing = 1

        config.master_url    = master_url or os.path.join(options.html_url , self.short_name , '')
        config.download_url  = os.path.join(config.master_url, 'download')
        config.upload_url    = os.path.join(cgi_url     , 'file_upload_handler')
        config.download_dir  = os.path.join(self.project_dir , 'download')
        config.upload_dir    = os.path.join(self.project_dir , 'upload')
        config.key_dir       = key_dir or os.path.join(self.project_dir , 'keys')
        config.app_dir       = os.path.join(self.project_dir, 'apps')
        config.log_dir       = self.project_dir+'log_'+config.host
        if production:
            config.min_sendwork_interval = 6
        self.scheduler_url = os.path.join(cgi_url     , 'cgi')

    def dest(self, *dirs):
        location = self.project_dir
        for x in dirs:
            location = os.path.join(location, x)
        return location

    def keydir(self, location):
            return os.path.join(self.config.config.key_dir, location)

    def logdir(self):
        return os.path.join(self.project_dir, "log_"+self.config.config.host)

    def create_keys(self):
        if not os.path.exists(self.config.config.key_dir):
            os.mkdir(self.config.config.key_dir)
        _gen_key(self.keydir('upload'))
        _gen_key(self.keydir('code_sign'))

    def create_logdir(self):
        os.mkdir(self.logdir())
        os.chmod(self.logdir(), 0o2770)

    def keys_exist(self):
        keys = ['upload_private', 'upload_public',
                'code_sign_private', 'code_sign_public' ]
        for key in keys:
            if not os.path.exists(self.keydir(key)): return False
        return True

    # create new project.  Called only from make_project
    def install_project(self):
        if os.path.exists(self.dest()):
            raise SystemExit('Project directory "%s" already exists; this would clobber it!'%self.dest())

        verbose_echo(1, "Creating directories")

        create_project_dirs(self.project_dir)

        if not self.web_only:
            if not self.keys_exist():
                verbose_echo(1, "Generating encryption keys")
                self.create_keys()

        # copy the user and administrative PHP files to the project dir,
        verbose_echo(1, "Copying files")

        # Create the project log directory
        self.create_logdir()

        install_boinc_files(self.dest(), True, not self.web_only)

        # copy sample web files to final names
        install(srcdir('html/user/sample_index.php'),
            self.dest('html/user/index.php'))
        install(srcdir('html/user/sample_bootstrap.min.css'),
            self.dest('html/user/bootstrap.min.css'))
        install(srcdir('html/user/sample_bootstrap.min.js'),
            self.dest('html/user/bootstrap.min.js'))
        install(srcdir('html/user/sample_jquery.min.js'),
            self.dest('html/user/jquery.min.js'))
        install(srcdir('html/project.sample/project.inc'),
            self.dest('html/project/project.inc'))
        install(srcdir('html/project.sample/project_specific_prefs.inc'),
            self.dest('html/project/project_specific_prefs.inc'))
        install(srcdir('html/project.sample/cache_parameters.inc'),
            self.dest('html/project/cache_parameters.inc'))
        install(srcdir('tools/project.xml'), self.dest('project.xml'))
        install(srcdir('tools/gui_urls.xml'), self.dest('gui_urls.xml'))
        if not self.production:
            install(srcdir('test/uc_result'), self.dest('templates/uc_result'))
            install(srcdir('test/uc_wu_nodelete'), self.dest('templates/uc_wu'))

        content = '''
<!-- <scheduler>{url}</scheduler> -->
<link rel="boinc_scheduler" href="{url}">
        '''.format(url=self.scheduler_url.strip())
        f = open(self.dest('html/user', 'schedulers.txt'), 'w')
        f.write(content)
        f.close()

        if self.no_db:
            verbose_echo(1, "Not setting up database (--no_db was specified)")
        else:
            verbose_echo(1, "Setting up database")
            database.create_database(
                srcdir = options.srcdir,
                config = self.config.config,
                drop_first = options.drop_db_first
                )

        verbose_echo(1, "Writing config files")
        self.config.write()

        # create symbolic links to the CGI and HTML directories
        verbose_echo(1, "Linking CGI programs")
        if options.__dict__.get('cgi_dir'):
            force_symlink(self.dest('cgi-bin'), os.path.join(options.cgi_dir, self.short_name))
        if options.__dict__.get('html_dir'):
            force_symlink(self.dest('html/user'), os.path.join(options.html_dir, self.short_name))
            force_symlink(self.dest('html/ops'), os.path.join(options.html_dir, self.short_name+'_admin'))

    def http_password(self, user, password):
        'Adds http password protection to the html/ops directory'
        passwd_file = self.dest('html/ops', '.htpassword')
        content = '''
AuthName '{long_name} Administration'
AuthType Basic
AuthUserFile {passwd_file}
require valid-user
'''.format(long_name=self.long_name, passwd_file=passwd_file)
        f = open(self.dest('html/ops', '.htaccess'), 'w')
        f.write(content)
        f.close()
        shell_call("htpassword -bc %s %s %s" % (passwd_file, user, password))

    def _run_sched_prog(self, prog, args='', logfile=None):
        verbose_shell_call("cd %s && ./%s %s >> %s.log 2>&1" %
                           (self.dest('bin'), prog, args, (logfile or prog)))

    def start_servers(self):
        self.started = True
        self._run_sched_prog('start', '-v --enable')
        verbose_sleep("Starting servers for project '%s'" % self.short_name, 1)

    def _build_sched_commandlines(self, progname, kwargs):
        '''Given a KWARGS dictionary build a list of command lines string depending on the program.'''
        each_app = False
        if progname == 'feeder':
            _check_vars(kwargs)
        elif progname == 'transitioner':
            _check_vars(kwargs)
        elif progname == 'make_work':
            work = kwargs.get('work', self.work)
            _check_vars(kwargs, cushion=30, max_wus=0, wu_name=work.wu_template)
        elif progname == 'sample_bitwise_validator':
            _check_vars(kwargs)
            each_app = True
        elif progname == 'file_deleter':
            _check_vars(kwargs)
        elif progname == 'antique_file_deleter':
            _check_vars(kwargs)
        elif progname == 'sample_dummy_assimilator':
            _check_vars(kwargs)
            each_app = True
        else:
            raise SystemExit("test script error: invalid progname '%s'"%progname)
        cmdline = build_command_line('', kwargs)
        if each_app:
            return [ '-app %s %s'%(av.app.name,cmdline) for av in self.app_versions ]
        else:
            return [cmdline]

    def sched_run(self, prog, **kwargs):
        for cmdline in self._build_sched_commandlines(prog, kwargs):
            self._run_sched_prog(prog, '-d 3 -one_pass '+cmdline)
    def sched_install(self, prog, **kwargs):
        for cmdline in self._build_sched_commandlines(prog, kwargs):
            self.config.daemons.make_node_and_append("daemon").cmd = "%s -d 3 %s" %(prog, cmdline)
        self.config.write()
    # def sched_uninstall(self, prog):
    #     self.config_daemons = XXX filter(lambda l: l.find(prog)==-1, self.config_daemons)
    #     self.config.write()

    def start_stripcharts(self):
        cgi_bin = [ 'stripchart.cgi', 'stripchart', 'stripchart.cnf',
            'looper', 'db_looper', 'datafiles', 'get_load', 'dir_size' ]
        for f in cgi_bin:
            self.copy(os.path.join('stripchart', f), 'cgi-bin/')

        macro_substitute('BOINC_DB_NAME', self.db_name, srcdir('stripchart/samples/db_count'),
                         self.dest('bin/db_count'))
        make_executable(self.dest('bin/db_count'))

        self._run_sched_prog('looper'    , 'get_load 1'                            , 'get_load')
        self._run_sched_prog('db_looper' , '"result" 1'                            , 'count_results')
        self._run_sched_prog('db_looper' , '"workunit where assimilate_state=2" 1' , 'assimilated_wus')
        self._run_sched_prog('looper'    , '"dir_size ../download" 1'              , 'download_size')
        self._run_sched_prog('looper'    , '"dir_size ../upload" 1'                , 'upload_size')

    def stop(self):
        verbose_echo(1,"Stopping server(s) for project '%s'"%self.short_name)
        self._run_sched_prog('start', '-v --disable')
        self.started = False

    def maybe_stop(self):
        if self.started: self.stop()

def query_noyes(str):
    verbose_echo(0,'')
    return tools.query_noyes(str)

def query_yesno(str):
    verbose_echo(0,'')
    return tools.query_yesno(str)



#!/usr/bin/env python

# projectxml.py - module to read and parse project.xml

'''
SYNOPSIS:  parses and writes project.xml
USAGE:     from Boinc import projectxml
           project = projectxml.ProjectFile().read()
           project.commit_all()
'''

import sys
from Boinc import boinc_project_path
from Boinc.boincxml import *
from Boinc.add_util import *

default_project_file = None

class ProjectFile(XMLConfig):
    '''
    embodies project.xml
    '''
    default_filename = boinc_project_path.project_xml_filename
    def __init__(self, *args, **kwargs):
        XMLConfig.__init__(self, *args, **kwargs)
        global default_project_file
        default_project_file = self
    def _get_elements(self):
        self.xml_boinc   = get_element(self.xml, 'boinc', optional=False)
        self.elements = ConfigDictList(self.xml_boinc)
        self.add_objects_and_args = []
        for node in self.xml_boinc.childNodes:
            add_object = add_objects.get(node.nodeName)
            if not add_object:
                raise SystemExit("Error in %s: No such object '%s' to add." %(self.filename,node.nodeName))
            self.add_objects_and_args.append((add_object, get_elements_as_dict(node)))
    def _set_elements(self):
        self.elements.save()
    def commit_all(self):
        '''Commits all new data to the BOINC project database.'''
        for add_object, untranslated_args_dict in self.add_objects_and_args:
            try:
                do_add_object(add_object, untranslated_args_dict, skip_old=True)
            except AddObjectException as e:
                raise SystemExit('Error in %s: %s' %(self.filename,e))
    default_xml = '<boinc></boinc>'

def default_project():
    '''If any project file has been read, return it.  Else open the default one
    and return it.'''
    if not default_project_file: ProjectFile().read()
    assert(default_project_file)
    return default_project_file



#!/usr/bin/env python3

# This file is part of BOINC.
# https://boinc.berkeley.edu
# Copyright (C) 2025 University of California
#
# BOINC is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# BOINC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with BOINC.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib
import sys

def help():
    print('Usage:')
    print('python trailing_whitespaces_check.py <dir>')

def check(directory, exclude_dirs, exclude_extensions, exclude_files, fix_errors=False):
    if not os.path.isdir(directory):
        return None

    files_with_errors = []

    for root, dirs, files in os.walk(directory, topdown=True):
        dirs[:] = [d for d in dirs if os.path.join(root, d) not in exclude_dirs]
        for filename in files:
            if (pathlib.Path(filename).suffix in exclude_extensions):
                continue
            src_file = os.path.join(root, filename)
            if (os.path.islink(src_file) or src_file in exclude_files or filename in exclude_files):
                continue
            with open(src_file, "rb") as f:
                if verbose:
                    print('Checking file: ' + src_file)
                lines = f.read().splitlines()
            count = 0
            changed = False
            for line in lines:
                count += 1
                if line.rstrip() != line:
                    if fix_errors:
                        lines[count - 1] = line.rstrip()
                        changed = True
                    else:
                        files_with_errors.append("{}:{}".format(src_file, count))
            if changed:
                print('Fixing file: ' + src_file + '...')
                with open(src_file, "wb") as f:
                    f.write(str.encode(os.linesep).join(lines))
                    f.write(str.encode(os.linesep))
    return files_with_errors


if (len(sys.argv) < 2):
    help()
    sys.exit(1)

verbose = False
fix = False
directory = sys.argv[1]

for arg in sys.argv:
    if arg == "--verbose":
        verbose = True
        continue
    if arg == "--fix":
        fix = True
        continue

exclude_dirs = [
    os.path.join(directory, ".git"),
    os.path.join(directory, ".github", "ISSUE_TEMPLATE"),
    os.path.join(directory, "3rdParty", "android"),
    os.path.join(directory, "3rdParty", "buildCache"),
    os.path.join(directory, "3rdParty", "linux"),
    os.path.join(directory, "3rdParty", "Windows"),
    os.path.join(directory, "3rdParty", "vcpkg_ports", "ports"),
    os.path.join(directory, "android", "BOINC", ".gradle"),
    os.path.join(directory, "android", "BOINC", ".idea"),
    os.path.join(directory, "android", "BOINC", "app", "build"),
    os.path.join(directory, "android", "BOINC", "app", "src", "main", "assets"),
    os.path.join(directory, "android", "BOINC", "app", "src", "main", "res"),
    os.path.join(directory, "api", "ttf"),
    os.path.join(directory, "coprocs", "NVIDIA", "include"),
    os.path.join(directory, "doc", "manpages"),
    os.path.join(directory, "html", "inc", "password_compat"),
    os.path.join(directory, "html", "inc", "random_compat"),
    os.path.join(directory, "html", "inc", "ReCaptcha"),
    os.path.join(directory, "locale"),
    os.path.join(directory, "mac_build", "build"),
    os.path.join(directory, "mac_build", "boinc.xcodeproj", "project.xcworkspace"),
    os.path.join(directory, "samples", "example_app", "bin"),
    os.path.join(directory, "samples", "gfx_html"),
    os.path.join(directory, "samples", "glut"),
    os.path.join(directory, "samples", "image_libs"),
    os.path.join(directory, "samples", "jpeglib"),
    os.path.join(directory, "samples", "mac_build", "build"),
    os.path.join(directory, "samples", "mac_build", "UpperCase2.xcodeproj", "project.xcworkspace"),
    os.path.join(directory, "samples", "vboxwrapper", "build"),
    os.path.join(directory, "samples", "vboxwrapper", "vboxwrapper.xcodeproj", "project.xcworkspace"),
    os.path.join(directory, "tests", "old"),
    os.path.join(directory, "win_build", "Build"),
    os.path.join(directory, "win_build", ".vs"),
    os.path.join(directory, "win_build", "ipch"),
    os.path.join(directory, "win_build", "installerv2", "redist"),
    os.path.join(directory, "zip", "unzip"),
    os.path.join(directory, "zip", "zip"),
    os.path.join(directory, "zip", "boinc_zip.xcodeproj", "project.xcworkspace"),
]

exclude_files = [
    ".DS_Store",
    os.path.join(directory, "bolt_checkin_notes.txt"),
    os.path.join(directory, "checkin_notes"),
    os.path.join(directory, "checkin_notes_2004"),
    os.path.join(directory, "checkin_notes_2005"),
    os.path.join(directory, "checkin_notes_2006"),
    os.path.join(directory, "checkin_notes_2007"),
    os.path.join(directory, "checkin_notes_2008"),
    os.path.join(directory, "checkin_notes_2009"),
    os.path.join(directory, "checkin_notes_2010"),
    os.path.join(directory, "checkin_notes_2011"),
    os.path.join(directory, "checkin_notes_2012"),
    os.path.join(directory, "checkin_notes_samples"),
    os.path.join(directory, "checkin_notes_samples"),
    os.path.join(directory, "todo"),
    os.path.join(directory, "TODO_OLD"),
    os.path.join(directory, "aclocal.m4"),
    os.path.join(directory, "api", "MultiGPUMig.h"),
    os.path.join(directory, "api", "MultiGPUMigServer.c"),
    os.path.join(directory, "api", "MultiGPUMigServer.h"),
    os.path.join(directory, "api", "MultiGPUMigUser.c"),
    os.path.join(directory, "client", "boinc_client"),
    os.path.join(directory, "client", "boinccmd"),
    os.path.join(directory, "client", "boinc"),
    os.path.join(directory, "client", "switcher"),
    os.path.join(directory, "clientscr", "progress", "simt"),
    os.path.join(directory, "html", "inc", "GeoIP.dat"),
    os.path.join(directory, "html", "inc", "geoip.inc"),
    os.path.join(directory, "html", "inc", "htmLawed.php"),
    os.path.join(directory, "html", "inc", "recaptcha_loader.php"),
    os.path.join(directory, "samples", "vboxwrapper", "x86_64", "vboxwrapper"),
    os.path.join(directory, "samples", "wrapper", "regerror.c"),
    os.path.join(directory, "samples", "wrapper", "regexp_custom.h"),
    os.path.join(directory, "samples", "wrapper", "regexp_int.h"),
    os.path.join(directory, "samples", "wrapper", "regexp_memory.c"),
    os.path.join(directory, "samples", "wrapper", "regexp_report.c"),
    os.path.join(directory, "samples", "wrapper", "regexp.c"),
    os.path.join(directory, "samples", "wrapper", "regexp.h"),
    os.path.join(directory, "samples", "wrapper", "regmagic.h"),
    os.path.join(directory, "samples", "wrapper", "regsub.c"),
    os.path.join(directory, "stage", "usr", "local", "bin", "boinc_client"),
    os.path.join(directory, "stage", "usr", "local", "bin", "boinccmd"),
    os.path.join(directory, "stage", "usr", "local", "bin", "boinc"),
    os.path.join(directory, "stage", "usr", "local", "bin", "switcher"),
]

exclude_extensions = [
    ".a",
    ".bmp",
    ".cub",
    ".dll",
    ".DS_Store",
    ".exe",
    ".gch",
    ".gif",
    ".icns",
    ".ico",
    ".jar",
    ".jpg",
    ".lib",
    ".mo",
    ".nib",
    ".o",
    ".odp",
    ".pdb",
    ".pdf",
    ".pdn",
    ".png",
    ".po",
    ".psd",
    ".rgb",
    ".rtf",
    ".so",
    ".tif",
    ".tiff",
    ".tlb",
    ".ttf",
    ".zip",
]

files_with_errors = check(directory, exclude_dirs, exclude_extensions, exclude_files, fix)

if files_with_errors:
    print('Found files with errors:')
    for file in files_with_errors:
        print(file)
    print('To fix errors run locally:\n\tpython3 ci_tools/trailing_whitespaces_check.py . --fix')
    sys.exit(1)

sys.exit(0)



#!/usr/bin/env python

## $Id$

# Make sure server hears that client exited with nonzero status.

from test_uc import *

class WorkExit(WorkUC):
    def __init__(self):
        WorkUC.__init__(self)
        self.wu_template = "uc_exit_wu"

class ResultExit(ResultUCError):
    def __init__(self):
        ResultUCError.__init__(self)
        self.stderr_out.append('<message>process exited with a non-zero exit code')

class ProjectExit(ProjectUC):
    def __init__(self):
        ProjectUC.__init__(self, short_name='test_exit', works=[WorkExit()])
    def check(self):
        self.check_client_error(ResultExit())

if __name__ == '__main__':
    test_msg("application exit report mechanism")
    ProjectExit()
    run_check_all()
